import { BasicEvent } from "@h4x/common";
import { ValidatorOutput } from "./CEValidator";
export declare enum Type {
    Unknown = "unknown",
    Boolean = "boolean",
    Number = "number",
    String = "string",
    Object = "object",
    Array = "array",
    Map = "map"
}
export interface ElementType {
    type: Type;
    optional?: boolean;
}
export interface BooleanType extends ElementType {
    type: Type.Boolean;
    default?: boolean;
}
export interface NumberType extends ElementType {
    type: Type.Number;
    default?: number;
    min?: number;
    max?: number;
}
export interface StringType extends ElementType {
    type: Type.String;
    default?: string;
    multiline?: boolean;
    options?: string[];
}
export interface ObjectType extends ElementType {
    type: Type.Object;
    properties: {
        [K: string]: ElementTypes;
    };
}
export interface ArrayType extends ElementType {
    type: Type.Array;
    elements: ElementTypes;
}
export interface MapType extends ElementType {
    type: Type.Map;
    elements: ElementTypes;
}
export declare type ElementTypes = BooleanType | NumberType | StringType | ObjectType | ArrayType | MapType;
export declare type ContainerTypes = ObjectType | ArrayType | MapType;
export declare namespace CEUtils {
    function getNew(schema: ElementTypes): any;
    function renderInput(schema: any, data: any, key: string, onUpdate: BasicEvent<any>): JSX.Element;
    function parseInfo(data: ValidatorOutput): JSX.Element[];
}
