import { BaseClass } from "@h4x/common";
export declare const options: {
    debug?: boolean;
    verbose?: boolean;
};
export declare class VNode<P = any> {
    type: (new () => Component<P>) | string;
    children: ComponentChild[];
    props: P;
    key?: string | number;
}
export declare type ComponentChild = VNode | string | number | null;
export declare type ComponentChildren = ComponentChild[] | ComponentChild;
export interface Attributes<T> {
    key?: string | number | any;
    ref?: (instance: T) => void;
}
export declare type ComponentProps<P, RefType = any> = Readonly<Attributes<RefType> & P>;
export declare function h<P>(node: (new () => Component<P>), props: Attributes<any> & P | null, ...children: ComponentChildren[]): VNode<P>;
export declare function h(node: string, props: JSX.HTMLAttributes<any> & JSX.SVGAttributes & Record<string, any> | null, ...children: ComponentChildren[]): VNode;
export declare function renderClass(Class: (new () => Component<any>), parent: Element | Document, before?: Node | number): Element | undefined;
export declare function render(vnode: ComponentChild, parent: Element | Document, before?: Node | number): Element | undefined;
export declare const $base: unique symbol;
declare const $updateProps: unique symbol;
declare const $update: unique symbol;
declare const $init: unique symbol;
declare const $attached: unique symbol;
declare const $detached: unique symbol;
declare const $destroy: unique symbol;
declare const $class: unique symbol;
export declare namespace TSX {
    export function configureInterval(fps: number): void;
    export type Props<T> = T extends Component<infer R> ? R : unknown;
    export class Rendered<T extends VComponent<any> | VDomNode = VComponent<any> | VDomNode> {
        readonly vnode: T;
        readonly element: Element;
        private __attached;
        get attached(): boolean;
        constructor(vnode: T, element: Element);
        attach(parent: Element | Document, before?: Node | number): void;
        detach(): void;
        /** @deprecated Internal */
        destroy(): void;
    }
    export type RenderedComponent<T extends Component<any>> = Rendered<VComponent<T>>;
    export type RenderedElement = Rendered<VDomElement | VComponent<any>>;
    export function prerender(vnode: VNode<any>): RenderedElement;
    export function prerender(vnode: string | number | null): Rendered<VDomNode>;
    export function prerender(vnode: ComponentChild): Rendered;
    export function prerenderComponent<T extends Component<any>>(vnode: ComponentChild): RenderedComponent<T>;
    export class Renderer<T extends (...args: any) => JSX.Element = () => JSX.Element> {
        private component;
        private fn;
        private args;
        private __rendered;
        get rendered(): boolean;
        readonly vnode: VComponent<any> | VDomNode;
        readonly element: Element;
        private constructor();
        update(): void;
        forceInstantUpdate(): void;
        private _persistent;
        get persistent(): boolean;
        private render;
        [$update](): void;
        /** @Internal */
        static renderNode(renderer: Renderer, persistent: boolean): any;
        /** @Internal */
        static attachNode(renderer: Renderer, vdom: VDomElement | VComponent<Component<any>>): void;
        /** @Internal */
        static detachNode(renderer: Renderer): void;
        static create<T extends (...args: any) => JSX.Element>(component: Component<any>, fn: T, args: FunctionArgs<T>): Renderer;
    }
    abstract class VDom {
        readonly key?: string | number;
        isActive: boolean;
        renderer?: Renderer;
        isCompatible(vnode: ComponentChild): boolean;
        abstract update(vnode: ComponentChild): void;
        attachRenderer(renderer: Renderer): void;
    }
    abstract class VDomNode extends VDom {
        protected nodeName: string;
        node: Node;
        children: (VDomNode | VComponent<any>)[];
        mappedChildren: Map<string | number, VComponent<any> | VDomNode>;
        inlined: Set<Renderer<() => JSX.Element>>;
        isPersistent: boolean;
        get typeName(): string;
        constructor();
        init(node: Node, key: string | number | undefined): this;
        abstract destroy(): void;
        addChild(vdom: VDomNode | VComponent<any>): void;
        updateChildren(vnode: VNode): void;
        setText(text: string): void;
    }
    class VDomElement extends VDomNode {
        readonly node: Element;
        private [$class];
        private props;
        destroy(): void;
        get typeName(): string;
        applyAttributes(target: any): any;
        private listeners;
        get eventHandler(): (this: Node, e: Event) => void;
        private applyAttribute;
        update(vnode: VNode): void;
        isCompatible(vnode: ComponentChild): boolean;
        setText(text: string): void;
    }
    export class VComponent<T extends Component<any>> extends VDom {
        instance: T;
        vdom: VDomElement | VComponent<any>;
        isPersistent: boolean;
        get node(): Node;
        constructor(type: new () => T, props: any, children: ComponentChild[]);
        destroy(): void;
        onAttached(): void;
        onDetached(): void;
        get typeName(): string;
        componentUpdate(vnode: VNode): void;
        update(vnode: VNode): void;
        isCompatible(vnode: ComponentChild): boolean;
    }
    export {};
}
declare type FunctionArgs<T extends (...args: any) => any> = T extends (...args: infer P) => any ? P : never;
export declare abstract class Component<P> extends BaseClass {
    static readonly Children: unique symbol;
    private static _REF;
    readonly REF: string;
    [$base]: TSX.VComponent<this>;
    private updateScheduled;
    private internalUpdating;
    private internalDestroyed;
    props: ComponentProps<P, this>;
    children: ComponentChild[];
    constructor();
    protected createRenderer<T extends (...args: any) => JSX.Element>(fn: T, ...args: FunctionArgs<T>): TSX.Renderer;
    abstract render(): VNode<this>;
    [$init](props: ComponentProps<P, any>, children: ComponentChild[]): void;
    protected init(): void;
    [$destroy](): void;
    protected destroy(): void;
    onAttached(): void;
    onDetached(): void;
    [$attached](): void;
    [$detached](): void;
    [$update](): void;
    protected update(): void;
    protected forceInstantUpdate(): void;
    protected preUpdate(): void;
    protected postUpdate(): void;
    protected receivedProps(props: ComponentProps<P, this>, children: ComponentChild[]): void;
    protected willReceiveProps(props: ComponentProps<P, this>, children: ComponentChild[]): void;
    [$updateProps](props: ComponentProps<P, this>, children: ComponentChild[]): void;
}
export declare namespace Component { }
export {};
