import { BaseClass, Utils, ClassVariable, RegisterClassVariable, BasicEvent, ObjectUtils, UniqueArray, ErrorResponse } from '@h4x/common';

const options = {};
// options.verbose = true;
function TSX_DEBUG(...args) {
    if (options.verbose === true) {
        console.warn(`[H4X-TSX-DEBUG] %c${args.shift()}`, "color: #00D; font-weight: 800;", ...args); // `
    }
}
function TSX_ERROR(...args) {
    console.warn(`[H4X-TSX-ERROR] %c${args.shift()}`, "color: #D00; font-weight: 800;", ...args); // `
}
class VNode {
}
function h(type, props, ...children) {
    let vnodes = [];
    parseChildren(typeof type !== "function", children, vnodes);
    let p = new VNode();
    p.type = type;
    p.children = vnodes;
    if (props !== null) {
        p.props = props;
        p.key = props.key;
    }
    return p;
}
function parseChildren(simple, children, output, lastSimple = false) {
    for (let i = 0; i < children.length; i++) {
        let child = children[i];
        if (Array.isArray(child)) {
            lastSimple = parseChildren(simple, child, output, lastSimple);
        }
        else {
            if (typeof child === "boolean") {
                child = null;
            }
            if (simple) {
                if (lastSimple) {
                    if (child === undefined || child === null) {
                        continue;
                    }
                    else if (typeof child === "number") {
                        output[output.length - 1] += "" + child;
                    }
                    else if (typeof child === "string") {
                        output[output.length - 1] += child;
                    }
                    else {
                        lastSimple = false;
                        output.push(child);
                    }
                }
                else {
                    output.push(child);
                }
            }
            else {
                output.push(child);
            }
        }
    }
    return lastSimple;
}
function renderClass(Class, parent, before) {
    return render(h(Class, null, []), parent, before);
}
function render(vnode, parent, before) {
    let rendered = TSX.prerender(vnode);
    rendered.attach(parent, before);
    if (parent instanceof Element && parent.isConnected) {
        if (parent.id === "app") {
            parent.classList.add("haxt");
        }
    }
    return rendered.element;
}
const $base = Symbol.for("$TSX_base");
const $updateProps = Symbol.for("$TSX_internal_UpdateProps");
const $update = Symbol.for("$TSX_internal_Update");
const $init = Symbol.for("$TSX_internal_Init");
const $attached = Symbol.for("$TSX_internal_Attached");
const $detached = Symbol.for("$TSX_internal_Detached");
const $destroy = Symbol.for("$TSX_internal_Destroy");
const $class = Symbol.for("$TSX_internal_prop_class");
let updateSchedule = new Set();
var TSX;
(function (TSX) {
    let interval;
    function configureInterval(fps) {
        if (interval !== undefined) {
            clearInterval(interval);
        }
        interval = setInterval(() => {
            let updating = updateSchedule;
            updateSchedule = new Set();
            updating.forEach((component) => {
                try {
                    component[$update]();
                }
                catch (err) {
                    console.error("update", err);
                }
            });
        }, 1000 / fps); // 20 fps
    }
    TSX.configureInterval = configureInterval;
    configureInterval(20);
    class Rendered {
        constructor(vnode, element) {
            this.vnode = vnode;
            this.element = element;
            this.__attached = false; // TODO change to #attached, fix rollup errors
        }
        get attached() { return this.__attached; }
        attach(parent, before) {
            if (this.__attached === true) {
                if (this.vnode instanceof VComponent) ;
            }
            if (before !== undefined) {
                if (typeof (before) === "number") {
                    if (before <= 0) {
                        parent.prepend(this.element);
                    }
                    else if (before >= parent.children.length) {
                        parent.append(this.element);
                    }
                    else {
                        parent.insertBefore(this.element, parent.children[before]);
                    }
                }
                else {
                    parent.insertBefore(this.element, before);
                }
            }
            else {
                parent.append(this.element);
            }
            if (this.__attached === false) {
                if (this.vnode instanceof VComponent) {
                    this.vnode.onAttached();
                    this.__attached = true;
                }
            }
            // else onMoved?
        }
        detach() {
            if (this.__attached === true) {
                this.element.remove();
                if (this.vnode instanceof VComponent) {
                    this.vnode.onDetached();
                }
                this.__attached = false;
            }
        }
        /** @deprecated Internal */
        destroy() {
            if (this.__attached === true) {
                this.element.remove();
                this.vnode.destroy();
                this.__attached = false;
            }
        }
    }
    TSX.Rendered = Rendered;
    function prerender(vnode) {
        let rendered = renderNode(vnode);
        if (rendered === undefined) {
            throw ("Couldn't render, got undefined!");
        }
        let element = rendered.node;
        return new Rendered(rendered, element);
    }
    TSX.prerender = prerender;
    function prerenderComponent(vnode) {
        if (!(vnode instanceof VNode) || typeof (vnode.type) === "string") {
            throw ("Couldn't prerenderComponent, not a Component!");
        }
        let rendered = renderNode(vnode);
        if (rendered === undefined) {
            throw ("Couldn't render, got undefined!");
        }
        let element = rendered.node;
        return new Rendered(rendered, element);
    }
    TSX.prerenderComponent = prerenderComponent;
    class Renderer {
        constructor(component, fn, args) {
            this.component = component;
            this.fn = fn;
            this.args = args;
            this.__rendered = false;
            this._persistent = false;
        }
        get rendered() { return this.__rendered; }
        update() {
            if (this.__rendered === true) {
                updateSchedule.add(this);
            }
        }
        forceInstantUpdate() {
            immediateUpdate(this);
        }
        get persistent() {
            return this._persistent;
        }
        render(persistent) {
            let rendered = this.fn.call(this.component, ...this.args);
            if (persistent !== undefined) {
                this._persistent = persistent;
            }
            return rendered;
        }
        [$update]() {
            if (this.__rendered === true) {
                this.vnode.update(this.render());
            }
        }
        /** @Internal */
        static renderNode(renderer, persistent) {
            renderer.__rendered = true;
            return renderer.render(persistent);
        }
        /** @Internal */
        static attachNode(renderer, vdom) {
            renderer.vnode = vdom;
            renderer.element = vdom.node;
        }
        /** @Internal */
        static detachNode(renderer) {
            if (renderer.persistent === false) {
                renderer.vnode = undefined;
                renderer.element = undefined;
            }
        }
        static create(component, fn, args) {
            return new Renderer(component, fn, args);
        }
    }
    TSX.Renderer = Renderer;
    // -------------------------------------------------------------------------------------------------------------- //
    const SVGElements = {
        // common
        svg: true, g: true, foreignObject: true,
        path: true, text: true, textPath: true,
        circle: true, ellipse: true, line: true, polygon: true, polyline: true, rect: true,
        // other
        animate: true, clipPath: true, defs: true, feBlend: true, feColorMatrix: true, feComponentTransfer: true,
        feComposite: true, feConvolveMatrix: true, feDiffuseLighting: true, feDisplacementMap: true, feFlood: true,
        feGaussianBlur: true, feImage: true, feMerge: true, feMergeNode: true, feMorphology: true, feOffset: true,
        feSpecularLighting: true, feTile: true, feTurbulence: true, filter: true, image: true, linearGradient: true,
        marker: true, mask: true, pattern: true, radialGradient: true, stop: true, symbol: true, tspan: true, use: true
    };
    function renderNode(vnode) {
        var _a;
        if (vnode instanceof VNode) {
            let vdom;
            let renderer;
            if (vnode.type === "inline") {
                renderer = vnode.props.renderer;
                if (renderer instanceof Renderer) {
                    let inlined = Renderer.renderNode(renderer, vnode.props.persistent);
                    if (inlined instanceof VNode) {
                        vnode = inlined;
                    }
                    else {
                        throw new Error("Invalid inline render!");
                    }
                }
                else {
                    throw new Error("Inline missing Renderer!");
                }
            }
            let type = vnode.type;
            if (typeof (type) === "string") {
                vdom = new VDomElement();
                if (SVGElements[type] === true) {
                    vdom.init(document.createElementNS("http://www.w3.org/2000/svg", type), vnode.key);
                }
                else {
                    vdom.init(document.createElement(type), vnode.key);
                }
                vdom.applyAttributes(vnode.props);
                if (vnode.type === "raw" || ((_a = vnode.props) === null || _a === void 0 ? void 0 : _a.raw) === true) {
                    if (vnode.children.length > 0) {
                        TSX_ERROR("[VDOM] Raw nodes shouldn't have children defined in VDOM!");
                    }
                }
                else {
                    for (const child of vnode.children) {
                        if (child instanceof VNode) {
                            let rendered = renderNode(child);
                            vdom.addChild(rendered);
                        }
                        else {
                            let rendered = renderNode(child);
                            if (rendered !== undefined) {
                                vdom.addChild(rendered);
                            }
                        }
                    }
                }
            }
            else {
                vdom = new VComponent(type, vnode.props, vnode.children);
            }
            if (renderer) {
                vdom.attachRenderer(renderer);
                Renderer.attachNode(renderer, vdom);
            }
            return vdom;
        }
        else if (typeof (vnode) === "string") {
            return new VDomText("" + vnode);
        }
        else if (typeof (vnode) === "number") {
            return new VDomText("" + vnode);
        }
        else if (vnode === undefined || vnode === null) { // FIXME remove null! throw?
            return undefined;
        }
        else if (Array.isArray(vnode)) {
            return undefined;
        }
        throw new Error("Invalid node type!");
    }
    class VDom {
        constructor() {
            this.isActive = true;
        }
        isCompatible(vnode) {
            throw new Error("Invalid isCompatible!");
        }
        attachRenderer(renderer) {
            this.renderer = renderer;
        }
    }
    class VDomNode extends VDom {
        constructor() {
            super();
            this.children = [];
            this.mappedChildren = new Map();
            this.inlined = new Set();
            this.isPersistent = false;
        }
        get typeName() {
            return "VDomNode";
        }
        init(node, key) {
            this.node = node;
            this.key = key;
            this.nodeName = this.node.nodeName.toLowerCase();
            return this;
        }
        addChild(vdom) {
            this.children.push(vdom);
            if (vdom.key !== undefined) {
                this.mappedChildren.set(vdom.key, vdom);
            }
            this.node.appendChild(vdom.node);
            if (vdom instanceof VComponent) {
                vdom.onAttached();
            }
            if (vdom.renderer) {
                this.inlined.add(vdom.renderer);
            }
        }
        updateChildren(vnode) {
            var _a;
            if (this.nodeName === "raw" || ((_a = vnode.props) === null || _a === void 0 ? void 0 : _a.raw) === true) {
                if (this.children.length > 0) {
                    TSX_ERROR("[VDOM] Raw nodes shouldn't have children defined in VDOM!");
                }
                return;
            }
            for (const child of this.children) {
                child.isActive = false;
            }
            let updatedChildren = [];
            let updatedInline = new Set();
            for (const vChild of vnode.children) {
                let current;
                if (vChild instanceof VNode && vChild.type === "inline") {
                    let renderer = vChild.props.renderer;
                    if (renderer instanceof Renderer) {
                        updatedInline.add(renderer);
                        current = renderer.vnode;
                    }
                    else {
                        throw new Error("Inline missing Renderer!");
                    }
                    if (current !== undefined) {
                        updatedChildren.push(current);
                        current.isActive = true;
                        continue;
                    }
                }
                else if (vChild instanceof VNode && vChild.key !== undefined) {
                    current = this.mappedChildren.get(vChild.key);
                    if (current !== undefined && current.isCompatible(vChild) === false) {
                        TSX_ERROR(`[VNode] Incompatible component with key ${JSON.stringify(vChild.key)}, got ${current.typeName}, expected ${vChild.type.name}!`); // `
                        let index = this.children.indexOf(current);
                        if (index > -1) {
                            this.children.splice(index, 1);
                            this.node.removeChild(current.node);
                        }
                        current.destroy();
                        current = undefined;
                        this.mappedChildren.delete(vChild.key);
                    }
                }
                else {
                    let c = 0; // TODO optimize start
                    while (c < this.children.length) {
                        let child = this.children[c++];
                        if (child.renderer !== undefined) {
                            continue;
                        } // skip children with a renderer attached
                        if (child.key !== undefined || child.isActive === true || child.isCompatible(vChild) === false) {
                            continue;
                        }
                        current = child;
                        break;
                    }
                }
                if (current) {
                    updatedChildren.push(current);
                    current.isActive = true; // mark it as active
                    current.update(vChild);
                }
                else {
                    let newChild = renderNode(vChild);
                    if (newChild) {
                        updatedChildren.push(newChild);
                    }
                }
            }
            for (const inlined of this.inlined) {
                if (updatedInline.has(inlined) === false) {
                    let child = inlined.vnode;
                    if (child.isPersistent === false) {
                        if (child.key !== undefined) {
                            this.mappedChildren.delete(child.key);
                        }
                    }
                    let index = this.children.findIndex(x => x === child);
                    this.node.removeChild(child.node);
                    this.children[index] = undefined;
                    Renderer.detachNode(inlined);
                    if (inlined.persistent === false) {
                        child.destroy();
                    }
                }
            }
            this.inlined = updatedInline;
            for (let c = 0; c < this.children.length; c++) {
                const child = this.children[c];
                if (child !== undefined && child.isActive === false) {
                    if (child.isPersistent === false) {
                        if (child.key !== undefined) {
                            this.mappedChildren.delete(child.key);
                        }
                        this.node.removeChild(child.node);
                        this.children[c] = undefined;
                        child.destroy();
                    }
                    else {
                        this.node.removeChild(child.node);
                        if (child instanceof VComponent) {
                            child.onDetached();
                        }
                        this.children[c] = undefined;
                    }
                }
            }
            let i = 0;
            let oldChildren = this.children;
            this.children = updatedChildren;
            for (const newChild of this.children) {
                let oldChild;
                while (oldChild === undefined && i < oldChildren.length) {
                    oldChild = oldChildren[i++];
                }
                let existing = false;
                if (oldChild) {
                    if (oldChild !== newChild) {
                        this.node.insertBefore(newChild.node, oldChild.node);
                        for (let c = i; c < oldChildren.length; c++) {
                            if (oldChildren[c] === newChild) {
                                oldChildren[c] = undefined;
                                existing = true;
                                break;
                            }
                        }
                        i--; // ???
                    }
                    else {
                        existing = true;
                    }
                }
                else {
                    this.node.appendChild(newChild.node);
                }
                if (existing === false) {
                    if (newChild.key !== undefined) {
                        if (options.debug === true) {
                            let current = this.mappedChildren.get(newChild.key);
                            if (current !== undefined && current.isPersistent === false) {
                                TSX_ERROR(`[VDOM] Key ${JSON.stringify(newChild.key)} already exists while adding a new key!`);
                            }
                        }
                        this.mappedChildren.set(newChild.key, newChild);
                    }
                    if (newChild instanceof VComponent) {
                        newChild.onAttached();
                    }
                }
            }
            if (options.debug === true) {
                let domNodes = Array.from(this.node.childNodes.values());
                let currentNodes = this.children.map(x => x.node);
                if (Utils.equals(domNodes, currentNodes) === false) {
                    TSX_ERROR("[VDOM] Update failed, child nodes don't match!");
                }
            }
        }
        setText(text) { throw new Error("Can't set text on non VDomText!"); }
    }
    const attributeHandlers = {
        class: (node, old, target) => {
            if (target === undefined || target === null) {
                node.removeAttribute("class");
            }
            else if (typeof (target) === "object") {
                if (options.debug === true) {
                    if (Array.isArray(target)) {
                        TSX_DEBUG("Got invalid attribute for class, of type array, and value " + target + "");
                    }
                    if (Array.isArray(old)) {
                        TSX_DEBUG("Got invalid attribute for class, of type array, and value " + target + "");
                    }
                }
                let classList = node.classList;
                if (old !== undefined && old !== null) {
                    // clear old values
                    for (const name in old) {
                        if (old[name] === true) {
                            const newValue = target[name];
                            if (newValue === undefined || newValue === false) {
                                classList.remove(name);
                            }
                        }
                    }
                    // apply/replace values
                    for (const name in target) {
                        const oldValue = old[name];
                        const newValue = target[name];
                        if (newValue === true && (oldValue === undefined || oldValue === false)) {
                            classList.add(name);
                        }
                    }
                }
                else {
                    // apply new values
                    for (const name in target) {
                        if (target[name] === true) {
                            classList.add(name);
                        }
                    }
                }
            }
            else {
                TSX_DEBUG("Got invalid attribute for class, of type " + (typeof (target)) + ", and value " + target + "");
            }
        },
        ref: (node, old, target) => { if (old !== target) {
            if (old) {
                old(null);
            }
            if (target) {
                target(node);
            }
        } },
        style: (node, old, target) => {
            let style = node.style;
            if (style instanceof CSSStyleDeclaration) {
                if (!target || typeof target === "string") {
                    style.cssText = target || "";
                }
                else if (typeof old === "string") {
                    style.cssText = "";
                }
                if (target && typeof target === "object") {
                    if (typeof old !== "string") {
                        for (let i in old) {
                            if (!(i in target)) {
                                style[i] = "";
                            }
                        }
                    }
                    for (let i in target) {
                        if (target.hasOwnProperty(i)) {
                            if (i === "flex") {
                                style[i] = target[i];
                            }
                            else {
                                style[i] = typeof target[i] === "number" ? (target[i] + "px") : target[i];
                            }
                        }
                    }
                }
            }
        },
        dangerouslySetInnerHTML: (node, _old, target) => { if (target) {
            node.innerHTML = target.__html || "";
        } },
        children: () => { },
        innerHTML: () => { },
        raw: () => { },
    };
    attributeHandlers.className = attributeHandlers.class;
    class VDomElement extends VDomNode {
        constructor() {
            super(...arguments);
            this.props = {};
            this.listeners = {};
        }
        destroy() {
            for (const child of this.children) {
                child.destroy();
            }
            this.applyAttributes(undefined);
            this.node = undefined;
            this.props = undefined;
        }
        get typeName() {
            return "VDomElement->" + this.nodeName;
        }
        applyAttributes(target) {
            let dom = this.node;
            if (!target) {
                // remove all props
                for (let name in this.props) {
                    if (this.props.hasOwnProperty(name)) {
                        let old = name === "class" ? this[$class] : this.props[name];
                        this.applyAttribute(dom, name, old, undefined);
                    }
                }
                target = {};
            }
            else {
                // remove missing props
                for (let name in this.props) {
                    if (target[name] == null && this.props[name] != null) {
                        let old = name === "class" ? this[$class] : this.props[name];
                        this.applyAttribute(dom, name, old, undefined);
                    }
                }
                // add new props and update existing ones
                for (let name in target) {
                    if (target.hasOwnProperty(name)) {
                        let old = this.props[name];
                        // get value/checked directly from the dom
                        if (name === "value" || name === "checked") {
                            old = dom[name];
                        }
                        let value = target[name];
                        if (value !== old) {
                            if (name === "class") {
                                if (typeof (value) === "string") {
                                    value = value.split(" ").filter(x => x.length > 0);
                                    let converted = {};
                                    for (const name of value) {
                                        converted[name] = true;
                                    }
                                    value = converted;
                                }
                                else {
                                    if (Array.isArray(value)) {
                                        let converted = {};
                                        for (const name of value) {
                                            if (typeof (name) === "string" && name.length > 0) {
                                                converted[name] = true;
                                            }
                                        }
                                        value = converted;
                                    }
                                }
                                this.applyAttribute(dom, name, this[$class], value);
                                this[$class] = value;
                            }
                            else {
                                this.applyAttribute(dom, name, old, value);
                            }
                        }
                    }
                }
            }
            this.props = target;
        }
        get eventHandler() {
            let self = this;
            const fnBound = function (e) {
                if (self.listeners[e.type] !== undefined) {
                    self.listeners[e.type](e, this);
                }
            };
            Object.defineProperty(this, "eventHandler", { value: fnBound, configurable: true, writable: true });
            return fnBound;
        }
        applyAttribute(node, name, old, target) {
            let handler = attributeHandlers[name];
            if (handler !== undefined) {
                return handler(node, old, target);
            }
            if (name.startsWith("on")) {
                name = name.toLowerCase().substring(2);
                if (target) {
                    if (!old) {
                        node.addEventListener(name, this.eventHandler, false);
                    }
                }
                else {
                    node.removeEventListener(name, this.eventHandler, false);
                }
                this.listeners[name] = target;
            }
            else if (name.startsWith("capture")) {
                name = name.toLowerCase().substring(7);
                if (target) {
                    if (!old) {
                        node.addEventListener(name, this.eventHandler, true);
                    }
                }
                else {
                    node.removeEventListener(name, this.eventHandler, true);
                }
                this.listeners[name] = target;
            }
            else if (node.namespaceURI !== "http://www.w3.org/2000/svg"
                && name !== "href" && name !== "list" && name !== "type" && name !== "tabIndex" && name !== "download"
                && name in node) {
                try {
                    node[name] = (target == null) ? "" : target;
                }
                catch (e) { }
                if (target == null) {
                    node.removeAttribute(name);
                }
            }
            else {
                if (target == null) {
                    node.removeAttribute(name);
                }
                else if (typeof target !== "function" && typeof target !== "object") {
                    node.setAttribute(name, target);
                }
            }
        }
        update(vnode) {
            if (vnode instanceof VNode === false) {
                throw new Error("Invalid node type!");
            }
            let type = vnode.type;
            if (typeof (type) === "string") {
                this.applyAttributes(vnode.props);
                this.updateChildren(vnode);
            }
            else {
                throw new Error("Invalid node type!");
            }
        }
        isCompatible(vnode) {
            return vnode instanceof VNode && typeof vnode.type === "string" && vnode.type.toLowerCase() === this.nodeName;
        }
        setText(text) { throw new Error("Can't set text, invalid dom type!"); }
    }
    class VDomText extends VDomNode {
        constructor(text) {
            super();
            this.currentText = "";
            this.currentText = text;
            this.init(document.createTextNode(text), undefined);
        }
        destroy() {
            this.node = undefined;
        }
        get typeName() {
            return "VDomText";
        }
        setText(text) {
            if (this.currentText !== text) {
                this.currentText = text;
                this.node.nodeValue = text;
            }
        }
        update(vnode) {
            if (typeof (vnode) === "string") {
                this.setText(vnode);
            }
            else if (typeof (vnode) === "number") {
                this.setText("" + vnode);
            }
            else {
                throw new Error("Invalid node type!");
            }
        }
        isCompatible(vnode) {
            return typeof vnode === "string" || typeof vnode === "number";
        }
    }
    class VComponent extends VDom {
        constructor(type, props, children) {
            super();
            this.isPersistent = false;
            if (type.prototype === undefined || type.prototype.render === undefined) {
                throw new Error("Component need to have a render function!");
            }
            let instance = new type();
            if (!(type.prototype instanceof Component)) {
                throw new Error("Components must extend the Component class!");
            }
            this.instance = instance;
            instance[$base] = this;
            instance[$init](props, children);
            this.vdom = renderNode(instance.render());
            if (props !== undefined) {
                if (props.key) {
                    this.key = props.key;
                }
                if (props.persistent) {
                    this.isPersistent = true;
                }
                if (props.ref) {
                    props.ref(instance);
                }
            }
        }
        get node() {
            return this.vdom.node;
        }
        destroy() {
            this.vdom.destroy();
            this.onDetached();
            this.instance[$destroy]();
            this.instance[$base] = undefined;
        }
        onAttached() {
            this.instance[$attached]();
        }
        onDetached() {
            this.instance[$detached]();
        }
        get typeName() {
            return "VDomComponent->" + this.instance.constructor.name;
        }
        componentUpdate(vnode) {
            if (this.vdom.isCompatible(vnode)) {
                TSX_DEBUG("Compatible", this, vnode);
                this.vdom.update(vnode);
            }
            else {
                // rerender
                TSX_DEBUG("Incompatible", this, vnode);
                let oldNode = this.node;
                this.vdom = renderNode(vnode);
                // debugger;
                oldNode.parentNode.replaceChild(this.node, oldNode);
            }
        }
        update(vnode) {
            TSX_DEBUG("VComponent update");
            if (vnode instanceof VNode === false) {
                throw new Error("Invalid node type!");
            }
            let type = vnode.type;
            if (typeof (type) === "string") {
                throw new Error("Invalid node type!");
            }
            else {
                let instance = this.instance;
                if (instance === undefined) {
                    throw new Error("Missing component instance!");
                }
                instance[$updateProps](vnode.props, vnode.children);
            }
        }
        isCompatible(vnode) {
            if (vnode instanceof VNode && typeof vnode.type === "function") {
                return this.instance.constructor === vnode.type;
            }
            return false;
        }
    }
    TSX.VComponent = VComponent;
})(TSX || (TSX = {}));
let immediateSchedule = new Set();
let immediate = false;
function immediateUpdate(component) {
    immediateSchedule.add(component);
    if (immediate === false) {
        immediate = true;
        Utils.nextTick().then(() => {
            let updating = immediateSchedule;
            immediateSchedule = new Set();
            updating.forEach((component) => {
                try {
                    component[$update]();
                }
                catch (err) {
                    console.error("immediateUpdate", err);
                }
            });
            immediate = false;
        });
    }
}
class Component extends BaseClass {
    constructor() {
        super();
        this.REF = "TSX:" + (Component._REF++);
        this.updateScheduled = false;
        this.internalUpdating = false;
        this.internalDestroyed = false;
        TSX_DEBUG("created " + this.constructor.name);
    }
    createRenderer(fn, ...args) {
        return TSX.Renderer.create(this, fn, args);
    }
    [$init](props, children) {
        if (props === undefined) {
            props = {};
        }
        this.props = props;
        this.children = children;
        if (this.__init) {
            this.__init();
        }
        this.init();
    }
    init() { }
    [$destroy]() {
        this.internalDestroyed = true;
        if (this.__destroy) {
            this.__destroy();
        }
        this.destroy();
    }
    destroy() { }
    onAttached() { }
    onDetached() { }
    [$attached]() { this.onAttached(); TSX_DEBUG("Attached " + this.REF); }
    [$detached]() { this.onDetached(); TSX_DEBUG("Detached " + this.REF); }
    [$update]() {
        if (this[$base] === undefined) ;
        else {
            this.preUpdate();
            this[$base].componentUpdate(this.render());
            this.postUpdate();
        }
    }
    update() {
        if (this[$base] === undefined) {
            if (this.internalDestroyed) {
                TSX_ERROR("Component dom is undefined when trying to schedule an update, the component was been already destroyed!");
            }
            else {
                TSX_ERROR("Component dom is undefined when trying to schedule an update!");
            }
        }
        if (this.internalUpdating) {
            this.updateScheduled = true;
        }
        else {
            updateSchedule.add(this);
        }
    }
    forceInstantUpdate() {
        if (this[$base] === undefined) {
            if (this.internalDestroyed) {
                TSX_ERROR("Component dom is undefined when trying to schedule an update, the component was been already destroyed!");
            }
            else {
                TSX_ERROR("Component dom is undefined when trying to schedule an update!");
            }
        }
        if (this.internalUpdating) {
            this.updateScheduled = true;
        }
        else {
            immediateUpdate(this);
        }
    }
    preUpdate() { }
    postUpdate() { }
    receivedProps(props, children) {
        TSX_DEBUG("receivedProps " + this.constructor.name);
        this.willReceiveProps(props, children);
        this.props = props;
        this.children = children;
        // if one of the update functions is called in receivedProps/willReceiveProps => this.updateScheduled = true;
        // this makes child updates instant when they are caused by props propagation
    }
    willReceiveProps(props, children) { }
    [$updateProps](props, children) {
        if (props === undefined) {
            props = {};
        }
        TSX_DEBUG("internalUpdateProps " + this.constructor.name);
        this.internalUpdating = true;
        this.receivedProps(props, children);
        this.internalUpdating = false;
        if (this.updateScheduled) {
            this[$update]();
            this.updateScheduled = false;
        }
    }
}
Component.Children = Symbol.for("$TSX_Children");
Component._REF = 1;

class IntervalVariable {
    constructor(time) {
        this.time = time;
    }
    register(variable) {
        this.name = variable.name;
    }
}
const INTERVAL = ClassVariable.createType("IntervalVariable");
class InitializeVariable {
    constructor() { }
    register(variable) {
        this.name = variable.name;
    }
}
const INITIALIZE = ClassVariable.createType("InitializeVariable");
class BasicComponent extends Component {
    constructor() {
        super();
        this.attachedEventReferences = [];
        this.onDestroy = new BasicEvent();
    }
    proxy(key) {
        return ObjectUtils.$(this, key);
    }
    $(key) {
        return ObjectUtils.$(this, key);
    }
    static var() {
        return (target, key) => {
            let internalVar = Symbol.for("var__" + key);
            Object.defineProperty(target, key, {
                get: function () {
                    return this[internalVar];
                },
                set: function (value) {
                    this[internalVar] = value;
                    this.$(key).onUpdate.execute(value);
                },
                enumerable: true
            });
        };
    }
    static bind() {
        return (target, key) => {
            if (target instanceof BasicComponent === false) {
                throw new Error("Failed to bind, class needs to be a subclass of BasicComponent!");
            }
            return ObjectUtils.bind()(target, key);
        };
    }
    static interval(time) {
        return function (target, key) {
            if (target instanceof BasicComponent === false) {
                throw new Error("Failed to attach interval, class needs to be a subclass of BasicComponent!");
            }
            RegisterClassVariable(target.constructor, key).attachData(INTERVAL, new IntervalVariable(time));
        };
    }
    static initialize() {
        return function (target, key) {
            if (target instanceof BasicComponent === false) {
                throw new Error("Failed to attach initialize, class needs to be a subclass of BasicComponent!");
            }
            RegisterClassVariable(target.constructor, key).attachData(INITIALIZE, new InitializeVariable());
        };
    }
    __init() {
        this[BaseClass.Meta].forEachVariable((variable, key) => {
            let intervalData = variable.getData(INTERVAL);
            if (intervalData !== undefined) {
                // intervalData.time
                let interval = window.setInterval(() => {
                    this[key]();
                }, intervalData.time);
                this.onDestroy.addCallback(() => {
                    if (interval !== undefined) {
                        clearInterval(interval);
                    }
                });
            }
            let initializeData = variable.getData(INITIALIZE);
            if (initializeData !== undefined) {
                this[key]();
            }
        });
    }
    __destroy() {
        this.onDestroy.execute();
        for (const ref of this.attachedEventReferences) {
            ref.remove();
        }
    }
    /** Attach event reference to be cleaned up when the object is destroyed */
    ref(ref) {
        this.attachedEventReferences.push(ref);
        return ref;
    }
    static normalizeClass(target) {
        if (Array.isArray(target)) {
            let out = {};
            for (const cls of target) {
                out[cls] = true;
            }
            return out;
        }
        else if (typeof target === "object") {
            return target;
        }
        else if (typeof target === "string") {
            let out = {};
            for (const cls of target.split(" ")) {
                out[cls] = true;
            }
            return out;
        }
        return undefined;
    }
    static mergeClasses(...target) {
        return Object.assign({}, ...target.map((cls) => BasicComponent.normalizeClass(cls)));
    }
}

class Overlay extends BasicComponent {
    constructor() {
        super();
        this.queue = [];
        this.alerts = [];
        this.floating = [];
    }
    static create(element) {
        if (this.instance) {
            console.error("Overlay already exists, overriding...");
        }
        element.classList.add("haxt");
        render(h(Overlay, { ref: (ref) => { this.instance = ref; } }), element);
    }
    static add(vnode) {
        let data = Overlay.createInstance(vnode);
        this.instance.queue.push(data);
        this.instance.update();
        return data;
    }
    static remove(component) {
        Overlay.removeInstance(this.instance.queue, component);
        this.instance.update();
    }
    static addAlert(vnode) {
        let data = Overlay.createInstance(vnode);
        this.instance.alerts.push(data);
        this.instance.update();
        return data;
    }
    static removeAlert(component) {
        Overlay.removeInstance(this.instance.alerts, component);
        this.instance.update();
    }
    static addFloating(vnode) {
        let data = Overlay.createInstance(vnode);
        this.instance.floating.push(data);
        this.instance.update();
        return data;
    }
    static removeFloating(component) {
        Overlay.removeInstance(this.instance.floating, component);
        this.instance.update();
    }
    postUpdate() {
        let count = this.queue.length;
        if (count > 0) {
            document.getElementsByTagName("body")[0].style.setProperty("overflow", "hidden");
        }
        else {
            document.getElementsByTagName("body")[0].style.setProperty("overflow", null);
        }
    }
    render() {
        let count = this.queue.length;
        if (count > 0) {
            let current = this.queue[count - 1];
            let output = current.vnode;
            output.props["key"] = output.key; // restore key
            return h("div", { id: "overlay-inner-container" },
                h("div", { class: "Overlay", key: "Overlay", style: { display: "flex" } }, output),
                h("div", { class: "OverlayAlerts", key: "OverlayAlerts" }, this.alerts.map((x) => x.vnode)),
                h("div", { class: "OverlayFloating", key: "OverlayFloating" }, this.floating.map((x) => x.vnode)));
        }
        else {
            return h("div", { id: "overlay-inner-container" },
                h("div", { class: "Overlay", key: "Overlay", style: { display: "none" } }),
                h("div", { class: "OverlayAlerts", key: "OverlayAlerts" }, this.alerts.map((x) => x.vnode)),
                h("div", { class: "OverlayFloating", key: "OverlayFloating" }, this.floating.map((x) => x.vnode)));
        }
    }
    /* Helper Functions */
    static createInstance(vnode) {
        if (this.instance === undefined) {
            console.error("Overlay not found...");
        }
        let data = { vnode: vnode, instance: undefined };
        if (vnode.props === undefined) {
            vnode.props = {};
        }
        vnode.props["ref"] = (ref) => {
            data.instance = ref;
        };
        vnode.key = "key_" + (Overlay.uniqueID++);
        vnode.props["persistent"] = true;
        return data;
    }
    static removeInstance(arr, component) {
        if (this.instance === undefined) {
            console.error("Overlay not found...");
        }
        const index = arr.findIndex(e => e.instance === component);
        let removed = undefined;
        if (index !== -1) {
            console.log("removing", index);
            removed = arr.splice(index, 1)[0];
        }
        if (removed) {
            removed.vnode.key = undefined;
            removed.vnode.props["persistent"] = undefined;
            if (removed.instance !== undefined) {
                removed.instance.__persistent = undefined;
                removed.instance[Symbol.for("$TSX_internal_Destroy")]();
            }
        }
    }
}
Overlay.uniqueID = 1;

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

function __decorate(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
}

function __metadata(metadataKey, metadataValue) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(metadataKey, metadataValue);
}

class BasicInput extends BasicComponent {
    init() {
        this.props.$.onUpdate.addCallback(this.update, this);
    }
    destroy() {
        this.props.$.onUpdate.removeCallback(this.update, this);
    }
    willReceiveProps() {
        this.update();
    }
}
class InputText extends BasicInput {
    onInput(event) {
        let value = event.target.value;
        this.props.$.set(value);
        this.props.$.onUpdate.execute(value);
    }
    onChange(event) {
        let value = event.target.value;
        this.props.$.onChange.execute(value);
    }
    render() {
        let _a = this.props, { $, ref } = _a, props = __rest(_a, ["$", "ref"]);
        return h("input", Object.assign({ type: "text" }, props, { value: $.get(), onInput: this.onInput, onChange: this.onChange }));
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputText.prototype, "onInput", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputText.prototype, "onChange", null);
class InputNumber extends BasicInput {
    onInput(event) {
        let value = +event.target.value;
        this.props.$.set(value);
        this.props.$.onUpdate.execute(value);
    }
    onChange(event) {
        let value = event.target.value;
        this.props.$.onChange.execute(value);
    }
    render() {
        let _a = this.props, { $, ref } = _a, props = __rest(_a, ["$", "ref"]);
        return h("input", Object.assign({ type: "number" }, props, { value: $.get(), onInput: this.onInput, onChange: this.onChange }));
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputNumber.prototype, "onInput", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputNumber.prototype, "onChange", null);
class InputRange extends BasicInput {
    onInput(event) {
        let value = +event.target.value;
        this.props.$.set(value);
        this.props.$.onUpdate.execute(value);
    }
    onChange(event) {
        let value = event.target.value;
        this.props.$.onChange.execute(value);
    }
    render() {
        let _a = this.props, { $, ref } = _a, props = __rest(_a, ["$", "ref"]);
        return h("input", Object.assign({ type: "range" }, props, { value: $.get(), onInput: this.onInput, onChange: this.onChange }));
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputRange.prototype, "onInput", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputRange.prototype, "onChange", null);
class InputTextBox extends BasicInput {
    onInput(event) {
        let value = event.target.value;
        this.props.$.set(value);
        this.props.$.onUpdate.execute(value);
    }
    onChange(event) {
        let value = event.target.value;
        this.props.$.onChange.execute(value);
    }
    onKeyDown(event, input) {
        if (event.key == "Tab") {
            event.preventDefault();
            var start = input.selectionStart;
            var end = input.selectionEnd;
            // set textarea value to: text before caret + tab + text after caret
            input.value = input.value.substring(0, start) +
                "\t" + input.value.substring(end);
            // put caret at right position again
            input.selectionStart = input.selectionEnd = start + 1;
            return true;
        }
        return false;
    }
    render() {
        let _a = this.props, { $, ref, handleTabs } = _a, props = __rest(_a, ["$", "ref", "handleTabs"]);
        if (handleTabs === true) {
            if (props.onKeyDown === undefined) {
                props.onKeyDown = this.onKeyDown;
            }
            else {
                let onKeyDown = props.onKeyDown;
                props.onKeyDown = (...args) => this.onKeyDown(...args) || onKeyDown(...args);
            }
        }
        return h("textarea", Object.assign({}, props, { value: $.get(), onInput: this.onInput, onChange: this.onChange }));
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputTextBox.prototype, "onInput", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputTextBox.prototype, "onChange", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [KeyboardEvent, HTMLTextAreaElement]),
    __metadata("design:returntype", void 0)
], InputTextBox.prototype, "onKeyDown", null);
class InputCheckbox extends BasicInput {
    onInput(event) {
        let value = event.target.checked;
        this.props.$.set(value);
        this.props.$.onUpdate.execute(value);
    }
    onChange(event) {
        let value = event.target.value;
        this.props.$.onChange.execute(value);
    }
    render() {
        let _a = this.props, { $, ref } = _a, props = __rest(_a, ["$", "ref"]);
        return h("input", Object.assign({ type: "checkbox" }, props, { checked: $.get(), onInput: this.onInput, onChange: this.onChange }));
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputCheckbox.prototype, "onInput", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputCheckbox.prototype, "onChange", null);
/*
Static "inputs"
*/
class BasicText extends BasicInput {
    render() {
        let _a = this.props, { $, ref } = _a, props = __rest(_a, ["$", "ref"]);
        return h("span", Object.assign({}, props), $.get());
    }
}
class FixedNumber extends BasicInput {
    render() {
        let _a = this.props, { $, digits, multiplier, prefix, postfix, ref } = _a, props = __rest(_a, ["$", "digits", "multiplier", "prefix", "postfix", "ref"]);
        let number = $.get();
        if (multiplier !== undefined) {
            number *= multiplier;
        }
        return h("span", Object.assign({}, props),
            prefix,
            number.toFixed(digits),
            postfix);
    }
}
class InputSelect extends BasicInput {
    onInput(event) {
        let value = event.target.value;
        this.props.$.set(value);
        this.props.$.onUpdate.execute(value);
    }
    onChange(event) {
        let value = event.target.value;
        this.props.$.onChange.execute(value);
    }
    render() {
        let _a = this.props, { $, values, ref } = _a, props = __rest(_a, ["$", "values", "ref"]);
        let current = $.get();
        let inputs = [];
        if (Array.isArray(values)) {
            inputs = values.map(key => h("option", { value: key, selected: key === current }, key));
        }
        else {
            inputs = [];
            for (const key in values) {
                if (values.hasOwnProperty(key)) {
                    const value = values[key];
                    inputs.push(h("option", { value: key, selected: key === current }, value));
                }
            }
        }
        return h("select", Object.assign({}, props, { onInput: this.onInput, onChange: this.onChange }), inputs);
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputSelect.prototype, "onInput", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputSelect.prototype, "onChange", null);
/*
Specialized inputs
*/
class InputFile extends BasicComponent {
    onChange(event) {
        let value = event.target.files != null ? event.target.files[0] : undefined;
        this.props.onFile(value);
    }
    render() {
        let _a = this.props, props = __rest(_a, ["ref"]);
        return h("input", Object.assign({ type: "file" }, props, { onChange: this.onChange })); // FIXME
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputFile.prototype, "onChange", null);
class InputFiles extends BasicComponent {
    onChange(event) {
        let value = event.target.files != null ? Array.from(event.target.files) : undefined;
        this.props.onFiles(value);
    }
    render() {
        let _a = this.props, props = __rest(_a, ["ref"]);
        return h("input", Object.assign({ type: "file" }, props, { onChange: this.onChange, multiple: true })); // FIXME
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], InputFiles.prototype, "onChange", null);

function highlightJSON(value, tabs = 0, out = []) {
    let type = typeof value;
    if (value === null) {
        out.push(h("span", { class: "null" }, "null"));
    }
    else if (value === undefined) {
        out.push(h("span", { class: "undefined" }, "undefined"));
    }
    else if (type === "number") {
        out.push(h("span", { class: "number" }, +value));
    }
    else if (type === "string") {
        out.push(h("span", { class: "string" },
            "\"",
            value,
            "\""));
    }
    else if (type === "boolean") {
        out.push(h("span", { class: "" + value }, "" + value));
    }
    else if (value.constructor === Object) {
        out.push("{\n");
        tabs += 1;
        let first = true;
        for (const key in value) {
            if (value.hasOwnProperty(key)) {
                const element = value[key];
                out.push(first ? ("\t".repeat(tabs)) : (",\n" + "\t".repeat(tabs)));
                out.push("\"");
                out.push(h("span", { class: "key" }, key));
                out.push("\": ");
                highlightJSON(element, tabs, out);
                first = false;
            }
        }
        tabs -= 1;
        out.push("\n" + "\t".repeat(tabs) + "}");
    }
    else if (value.constructor === Array) {
        out.push("[\n");
        tabs += 1;
        let isNumericArray = value.every(x => typeof x === "number");
        if (isNumericArray) {
            for (let i = 0; i < value.length; i++) {
                const element = value[i];
                out.push(i !== 0 ? ", " : "\t".repeat(tabs));
                highlightJSON(element, tabs, out);
            }
        }
        else {
            for (let i = 0; i < value.length; i++) {
                const element = value[i];
                out.push(i !== 0 ? ",\n" + "\t".repeat(tabs) : "\t".repeat(tabs));
                highlightJSON(element, tabs, out);
            }
        }
        tabs -= 1;
        out.push("\n" + "\t".repeat(tabs) + "]");
    }
    return out;
}
class JSONView extends BasicComponent {
    willReceiveProps() {
        this.update();
    }
    render() {
        return (h("pre", { class: "JSONView", style: { flex: 1 } }, highlightJSON(this.props.json)));
    }
}

class Tooltip extends BasicComponent {
    render() {
        return (h("div", Object.assign({ class: "Tooltip" }, this.props), this.children));
    }
}
class TooltipProvider extends BasicComponent {
    willReceiveProps() {
        this.update();
    }
    onMouseEnter( /*event: MouseEvent*/) {
        let container = this[$base].node;
        let rect = container.getClientRects();
        this.tooltip = Overlay.addFloating(h(Tooltip, { style: {
                top: rect[0].top,
                left: rect[0].right + 5,
                position: "fixed"
            } }, this.props.renderTooltip()));
    }
    onMouseLeave() {
        if (this.tooltip) {
            Overlay.removeFloating(this.tooltip.instance);
        }
    }
    render() {
        return (h("div", Object.assign({ class: "TooltipProvider", onMouseEnter: this.onMouseEnter, onMouseLeave: this.onMouseLeave }, this.props), this.children));
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TooltipProvider.prototype, "onMouseEnter", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TooltipProvider.prototype, "onMouseLeave", null);

class ListContainer {
    constructor(...init) {
        this.internal = [];
        this.onChanged = new BasicEvent();
        this.onRemove = new BasicEvent();
        this.internal = [...init];
    }
    get size() {
        return this.internal.length;
    }
    set(list) {
        this.internal = list;
        this.onChanged.execute();
    }
    get(index) {
        return this.internal[index];
    }
    add(value) {
        let ret = this.internal.push(value);
        this.onChanged.execute();
        return ret;
    }
    remove(index) {
        let removed = this.internal.splice(index, 1)[0];
        this.onChanged.execute();
        this.onRemove.execute(removed, index);
        return this.internal.length;
    }
    removeValue(value) {
        let index = this.internal.indexOf(value);
        if (index === -1) {
            return this.internal.length;
        }
        return this.remove(index);
    }
    each(callback) {
        this.internal.forEach(callback);
    }
    map(callback) {
        return this.internal.map(callback);
    }
}

class ListElement extends BasicComponent {
    constructor() {
        super(...arguments);
        this.selected = false;
    }
    init() {
        this.options = this.props.options;
        {
            let index = this.options.index;
            this.selected = (index === this.props.index);
        }
        this.options.onSelected.addCallback(this.checkSelected, this);
    }
    willReceiveProps(nextProps) {
        if (this.props.name !== nextProps.name) {
            this.update();
        }
        if (this.options !== nextProps.options) {
            this.options.onSelected.removeCallback(this.checkSelected, this);
            nextProps.options.onSelected.addCallback(this.checkSelected, this);
        }
    }
    checkSelected() {
        let index = this.options.index;
        let isSelected = (index === this.props.index);
        if (this.selected !== isSelected) {
            this.selected = isSelected;
            this.update();
        }
    }
    select() {
        this.options.onSelected.execute(this.props.index);
    }
    render() {
        let labelClass = this.props !== undefined ? this.props.labelClass : undefined;
        let clazz = {};
        clazz["ListElement"] = true;
        if (typeof (labelClass) === "string") {
            clazz = Object.assign(Object.assign({}, clazz), { [labelClass]: true });
        }
        else if (labelClass !== undefined) {
            clazz = Object.assign(Object.assign({}, clazz), labelClass);
        }
        if (this.selected === true) {
            clazz["selected"] = true;
        }
        return h("div", { key: this.props.name, class: clazz, onClick: this.select }, this.props.name);
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ListElement.prototype, "select", null);
class SelectContainer extends ListContainer {
    constructor(...init) {
        super(...init);
        this.onSelected = new BasicEvent();
        this.currentIndex = -1;
        this.onSelected.addCallback((index) => {
            this.currentIndex = index;
        });
        this.onRemove.addCallback((_value, index) => {
            if (index === this.currentIndex) {
                this.prev();
            }
            else if (index < this.currentIndex) {
                this.currentIndex--;
            }
        });
    }
    get index() {
        return this.currentIndex;
    }
    linkValue(variable, field) {
        this.onSelected.addCallback((index) => {
            variable[field] = this.get(index);
        });
        return true;
    }
    update(index) {
        if (this.currentIndex !== index) {
            this.currentIndex = index;
            this.onSelected.execute(index);
        }
    }
    prev() {
        if (this.currentIndex - 1 >= 0) {
            this.currentIndex--;
            this.onSelected.execute(this.currentIndex);
        }
    }
    next() {
        if (this.currentIndex + 1 < this.size) {
            this.currentIndex++;
            this.onSelected.execute(this.currentIndex);
        }
    }
}
class ListSelect extends BasicComponent {
    constructor() {
        super(...arguments);
        this.val = 123;
        this.uuid = Math.random();
    }
    init() {
        this.options = this.props.options;
        this.props.options = undefined;
        this.options.onChanged.addCallback(this.update, this);
    }
    willReceiveProps(nextProps) {
        if (this.options !== nextProps.options) {
            this.options = nextProps.options;
            if (this.options !== undefined) {
                this.options.onChanged.removeCallback(this.update, this);
            }
            nextProps.options = undefined;
            this.options.onChanged.addCallback(this.update, this);
            this.update();
        }
    }
    render() {
        return h("div", Object.assign({ class: "ListSelect" }, this.props), this.options.map((option, index) => {
            if (typeof (option) === "string") {
                return h(ListElement, { options: this.options, name: option, index: index });
            }
            else {
                return h(ListElement, { options: this.options, name: option.label, index: index, labelClass: option.labelClass });
            }
        }));
    }
}

const $remove = Symbol.for("$TSX_Fragment_Remove");
const $rendered = Symbol.for("$TSX_Fragment_Rendered");
const $attach = Symbol.for("$TSX_Fragment_Attach");
class Fragment extends BasicComponent {
    remove() {
        this.props.fragmentList[$remove](this[$rendered]);
        this[$rendered].destroy();
    }
    /*public moveTo(pos: number) {
        this.props.fragmentList.moveFragment(this[Rendered], pos);
    }

    public moveUp() {
        let index = this.index();
        this.props.fragmentList.moveFragment(this[Rendered], index - 1);
    }

    public moveDown() {
        let index = this.index();
        // +2 as it need to move over itself and the next element
        this.props.fragmentList.moveFragment(this[Rendered], index + 2);
    }

    public index() {
        return this.props.fragmentList.indexOf(this[Rendered]);
    }*/
    onDetached() {
    }
}
(function (Fragment) {
    class AbstractList {
        *instances() {
            for (const fragment of this) {
                yield fragment.vnode.instance;
            }
        }
        createFragment(FragmentClass, args) {
            let fragment = h(FragmentClass, Object.assign({}, args, { fragmentList: this }));
            let rendered = TSX.prerenderComponent(fragment);
            let instance = rendered.vnode.instance;
            instance[$rendered] = rendered;
            return { rendered, instance };
        }
        [$attach](component) {
            if (this.component !== undefined) {
                throw new Error("FragmentList already attached to a component");
            }
            this.component = component;
            for (const fragment of this) {
                this.component.attachFragment(fragment);
            }
        }
    }
    Fragment.AbstractList = AbstractList;
    class List extends AbstractList {
        constructor() {
            super(...arguments);
            this.fragments = new UniqueArray();
        }
        get length() { return this.fragments.length; }
        get first() { return this.fragments[0]; }
        get firstInstance() { var _a; return (_a = this.fragments[0]) === null || _a === void 0 ? void 0 : _a.vnode.instance; }
        get last() { return this.fragments[this.fragments.length - 1]; }
        get lastInstance() { var _a; return (_a = this.fragments[this.fragments.length - 1]) === null || _a === void 0 ? void 0 : _a.vnode.instance; }
        *[Symbol.iterator]() {
            for (const fragment of this.fragments) {
                yield fragment;
            }
        }
        get(at) {
            return this.fragments[at];
        }
        getInstance(at) {
            return this.fragments[at].vnode.instance;
        }
        indexOf(rendered) {
            if (rendered instanceof TSX.Rendered) {
                return this.fragments.indexOf(rendered);
            }
            else {
                return this.fragments.indexOf(rendered[$rendered]);
            }
        }
        /** Add fragment before `at` or at the end */
        addFragment(FragmentClass, args, at) {
            let { rendered, instance } = this.createFragment(FragmentClass, args);
            if (typeof (at) === "object") {
                let index = this.indexOf(at[$rendered]);
                if (index >= 0) {
                    at = index;
                }
                else {
                    console.warn("Couldn't find 'at' fragment when trying to add fragment!");
                    at = undefined;
                }
            }
            if (at === 0) {
                this.fragments.unshift(rendered);
            }
            else if (at !== undefined) {
                this.fragments.insertAt(at, rendered);
            }
            else {
                this.fragments.push(rendered);
            }
            if (this.component !== undefined) {
                this.component.attachFragment(rendered, at);
            }
            return instance;
        }
        moveFragment(rendered, to) {
            if (to !== undefined) {
                this.fragments.insertAt(to, rendered);
            }
            else if (to === 0) {
                this.fragments.unshift(rendered);
            }
            else {
                this.fragments.push(rendered);
            }
            if (this.component !== undefined) {
                this.component.attachFragment(rendered, to);
            }
        }
        /*
            FragmentList needs to have a way to move Fragments around (and to sort them?)
            swap? swap range? move range?
        */
        detachFragment(rendered) {
            if (this.fragments.removeElement(rendered)) {
                rendered.detach();
                return true;
            }
            return false;
        }
        detachAll() {
            for (const fragment of this.fragments) {
                fragment.detach();
            }
            this.fragments.clear();
        }
        destroyAll() {
            for (const fragment of this.fragments) {
                fragment.destroy();
            }
            this.fragments.clear();
        }
        [$attach](component) {
            super[$attach](component);
        }
        /** internal */
        [$remove](rendered) {
            if (this.fragments.removeElement(rendered)) {
                return true;
            }
            return false;
        }
    }
    Fragment.List = List;
    class Single extends AbstractList {
        get length() { return this.fragment ? 1 : 0; }
        get first() { return this.fragment; }
        get firstInstance() { var _a; return (_a = this.fragment) === null || _a === void 0 ? void 0 : _a.vnode.instance; }
        get last() { return this.fragment; }
        get lastInstance() { var _a; return (_a = this.fragment) === null || _a === void 0 ? void 0 : _a.vnode.instance; }
        *[Symbol.iterator]() {
            if (this.fragment) {
                yield this.fragment;
            }
        }
        indexOf(rendered) {
            if (rendered instanceof TSX.Rendered) {
                return this.fragment === rendered ? 0 : -1;
            }
            else {
                return this.fragment === rendered[$rendered] ? 0 : -1;
            }
        }
        set(FragmentClass, args) {
            if (this.fragment !== undefined) {
                this.fragment.destroy();
            }
            let { rendered, instance } = this.createFragment(FragmentClass, args);
            this.fragment = rendered;
            if (this.component !== undefined) {
                this.component.attachFragment(rendered);
            }
            return instance;
        }
        attach(rendered) {
            if (this.fragment === rendered) {
                return;
            }
            else if (this.fragment !== undefined) {
                this.fragment.destroy();
            }
            this.fragment = rendered;
            if (this.component !== undefined) {
                this.component.attachFragment(rendered);
            }
        }
        get() {
            return this.fragment;
        }
        get instance() {
            var _a;
            return (_a = this.fragment) === null || _a === void 0 ? void 0 : _a.vnode.instance;
        }
        detach() {
            if (this.fragment) {
                this.fragment.detach();
                this.fragment = undefined;
            }
        }
        destroy() {
            if (this.fragment) {
                this.fragment.destroy();
                this.fragment = undefined;
            }
        }
        /** internal */
        [$remove](rendered) {
            if (this.fragment === rendered) {
                this.fragment = undefined;
                return true;
            }
            return false;
        }
    }
    Fragment.Single = Single;
    class Container extends BasicComponent {
        attachContainer(instance) {
            if (this.container !== undefined) {
                if (this.container !== instance) {
                    throw new Error("Trying to reassign fragment container!");
                }
            }
            else {
                this.container = instance;
                this.props.fragmentList[$attach](this);
            }
        }
        attachFragment(rendered, at) {
            rendered.attach(this.container, at);
        }
        render() {
            const _a = this.props, { fragmentList, nodeType, key } = _a, props = __rest(_a, ["fragmentList", "nodeType", "key"]);
            let Type = nodeType !== null && nodeType !== void 0 ? nodeType : "raw";
            return h(Type, Object.assign({}, props, { ref: this.attachContainer, raw: true }));
        }
    }
    __decorate([
        BasicComponent.bind(),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [HTMLDivElement]),
        __metadata("design:returntype", void 0)
    ], Container.prototype, "attachContainer", null);
    Fragment.Container = Container;
})(Fragment || (Fragment = {}));

class ElementContainer extends BasicComponent {
    constructor() {
        super(...arguments);
        this.opened = true;
    }
    willReceiveProps() {
        this.update();
        this.initSchema();
    }
    initSchema() {
        let schema = this.props.properties;
        this.isBasic = false;
        if (schema.type === Type.Boolean || schema.type === Type.Number) {
            this.isBasic = true;
        }
        else if (schema.type === Type.String && schema.multiline !== true) {
            this.isBasic = true;
        }
    }
    toggle() {
        this.opened = !this.opened;
        this.update();
    }
    render() {
        let { properties, data, key, onUpdate } = this.props;
        if (this.isBasic) {
            return h("div", { style: { "margin": "3px 0" } },
                h("i", { style: { "margin-right": 13 } }),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } }, key),
                CEUtils.renderInput(properties, data, key, onUpdate));
        }
        else if (this.opened) {
            return h("div", { style: { "margin": "3px 0" } },
                h("i", { class: "fas fa-caret-down", style: { "vertical-align": "top", "width": 10, "text-align": "center", "margin-right": 3 }, onClick: this.toggle }),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } }, key),
                CEUtils.renderInput(properties, data, key, onUpdate));
        }
        else {
            return h("div", { style: { "margin": "3px 0" } },
                h("i", { class: "fas fa-caret-right", style: { "vertical-align": "top", "width": 10, "text-align": "center", "margin-right": 3 }, onClick: this.toggle }),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } }, key));
        }
    }
}
__decorate([
    BasicComponent.initialize(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ElementContainer.prototype, "initSchema", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ElementContainer.prototype, "toggle", null);
class ObjectContainer extends BasicComponent {
    willReceiveProps() {
        this.update();
    }
    render() {
        let { schema, data, onUpdate } = this.props;
        let inputs = [];
        let properties = schema.properties;
        for (const key in properties) {
            if (properties.hasOwnProperty(key)) {
                inputs.push(h(ElementContainer, { properties: properties[key], data: data, key: key, onUpdate: onUpdate }));
            }
        }
        return h("div", { style: { "padding-left": 10, "margin-left": 15 } }, inputs);
    }
}

class ElementContainer$1 extends BasicComponent {
    constructor() {
        super(...arguments);
        this.opened = true;
    }
    willReceiveProps() {
        this.update();
        this.initSchema();
    }
    initSchema() {
        let schema = this.props.properties;
        this.isBasic = false;
        if (schema.type === Type.Boolean || schema.type === Type.Number) {
            this.isBasic = true;
        }
        else if (schema.type === Type.String && schema.multiline !== true) {
            this.isBasic = true;
        }
    }
    delete() {
        this.props.data[this.props.key] = undefined;
        this.props.onDeleted();
        this.props.onUpdate.execute();
    }
    toggle() {
        this.opened = !this.opened;
        this.update();
    }
    render() {
        let { properties, data, key, onUpdate } = this.props;
        if (this.isBasic) {
            return h("div", { style: { "margin": "3px 0" } },
                h("i", { style: { "margin-right": 13 } }),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } }, key),
                h("i", { class: "far fa-trash-alt", style: { "color": "#C44", "margin-right": 5 }, onClick: this.delete }),
                CEUtils.renderInput(properties, data, key, onUpdate));
        }
        else if (this.opened) {
            return h("div", { style: { "margin": "3px 0" } },
                h("i", { class: "fas fa-caret-down", style: { "vertical-align": "top", "width": 10, "text-align": "center", "margin-right": 3 }, onClick: this.toggle }),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } }, key),
                h("i", { class: "far fa-trash-alt", style: { "color": "#C44" }, onClick: this.delete }),
                CEUtils.renderInput(properties, data, key, onUpdate));
        }
        else {
            return h("div", { style: { "margin": "3px 0" } },
                h("i", { class: "fas fa-caret-right", style: { "vertical-align": "top", "width": 10, "text-align": "center", "margin-right": 3 }, onClick: this.toggle }),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } }, key),
                h("i", { class: "far fa-trash-alt", style: { "color": "#C44" }, onClick: this.delete }));
        }
    }
}
__decorate([
    BasicComponent.initialize(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ElementContainer$1.prototype, "initSchema", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ElementContainer$1.prototype, "delete", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ElementContainer$1.prototype, "toggle", null);
class MapContainer extends BasicComponent {
    constructor() {
        super(...arguments);
        this.newName = "";
    }
    willReceiveProps() {
        this.update();
    }
    addNew() {
        if (this.newName.length <= 0) {
            throw new ErrorResponse("New element needs to have a name!");
        }
        if (this.props.data[this.newName] !== undefined) {
            throw new ErrorResponse("New element needs to have a unique name!");
        }
        this.props.data[this.newName] = CEUtils.getNew(this.props.schema.elements);
        this.newName = "";
        this.update();
        this.props.onUpdate.execute();
    }
    onDeleted() {
        this.update();
    }
    render() {
        let { schema, data, onUpdate } = this.props;
        let inputs = [];
        let properties = schema.elements;
        for (const key in data) {
            if (data.hasOwnProperty(key) && data[key] !== undefined) {
                inputs.push(h(ElementContainer$1, { properties: properties, data: data, key: key, onUpdate: onUpdate, onDeleted: this.onDeleted }));
            }
        }
        return h("div", { style: { "padding-left": 10, "margin-left": 15 } },
            inputs,
            h("div", null,
                h(InputText, { "$": this.$("newName") }),
                h("i", { class: "fas fa-plus-circle", style: { "color": "#4C4" }, onClick: this.addNew })));
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MapContainer.prototype, "addNew", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MapContainer.prototype, "onDeleted", null);

var ValidatorType;
(function (ValidatorType) {
    ValidatorType[ValidatorType["Error"] = 0] = "Error";
    ValidatorType[ValidatorType["Warning"] = 1] = "Warning";
    ValidatorType[ValidatorType["Info"] = 2] = "Info";
})(ValidatorType || (ValidatorType = {}));
class Validator {
    static boolean(schema, value, container, key) {
        let data = { errors: [], warnings: [], info: [] };
        Validator.required(data, schema, value);
        return data;
    }
    static text(schema, value, container, key) {
        let data = { errors: [], warnings: [], info: [] };
        Validator.required(data, schema, value);
        return data;
    }
    static number(schema, value, container, key) {
        let data = { errors: [], warnings: [], info: [] };
        Validator.required(data, schema, value);
        Validator.numberRange(data, schema, value);
        return data;
    }
    static numberRange(data, schema, value) {
        if (schema.min !== undefined && value < schema.min) {
            data.errors.push("The minimum value is " + schema.min);
        }
        if (schema.max !== undefined && value > schema.max) {
            data.errors.push("The maximum value is " + schema.max);
        }
    }
    static required(data, schema, value) {
        if (schema.optional !== true && value === undefined) {
            data.warnings.push("This property must have a value!");
        }
    }
}

class BooleanContainer extends BasicComponent {
    init() {
        this.validate();
    }
    willReceiveProps() {
        this.update();
    }
    get() {
        return this.props.data[this.props.key];
    }
    set(value) {
        this.props.data[this.props.key] = value;
    }
    onInput(event) {
        let value = event.target.checked === true;
        this.set(value);
        this.props.onUpdate.execute(value);
        this.validate();
    }
    validate() {
        let oldData = this.validatorData;
        this.validatorData = Validator.boolean(this.props.schema, this.get(), this.props.data, this.props.key);
        if (Utils.equals(oldData, this.validatorData) === false) {
            console.log("validate...");
            this.update();
        }
    }
    render() {
        let extra = CEUtils.parseInfo(this.validatorData);
        return h("div", { style: { "display": "inline-block" } },
            h("input", { class: "InputContainer BooleanContainer", type: "checkbox", checked: this.get(), onInput: this.onInput }),
            extra);
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], BooleanContainer.prototype, "onInput", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], BooleanContainer.prototype, "validate", null);

class NumberContainer extends BasicComponent {
    init() {
        this.validate();
    }
    willReceiveProps() {
        this.update();
    }
    get() {
        return this.props.data[this.props.key];
    }
    set(value) {
        this.props.data[this.props.key] = value;
    }
    onInput(event) {
        let value = event.target.value;
        if (value.length <= 0) {
            value = undefined;
        }
        else {
            value = +value;
        }
        this.set(value);
        this.props.onUpdate.execute(value);
        this.validate();
    }
    validate() {
        let oldData = this.validatorData;
        this.validatorData = Validator.number(this.props.schema, this.get(), this.props.data, this.props.key);
        if (Utils.equals(oldData, this.validatorData) === false) {
            console.log("validate...");
            this.update();
        }
    }
    render() {
        let extra = CEUtils.parseInfo(this.validatorData);
        return h("div", { style: { "display": "inline-block" } },
            h("input", { class: "InputContainer NumberContainer", type: "number", value: this.get(), onInput: this.onInput, onBlur: this.validate }),
            extra);
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], NumberContainer.prototype, "onInput", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], NumberContainer.prototype, "validate", null);

class TextContainer extends BasicComponent {
    init() {
        this.validate();
    }
    willReceiveProps() {
        this.update();
    }
    get() {
        return this.props.data[this.props.key];
    }
    set(value) {
        this.props.data[this.props.key] = value;
    }
    onInput(event) {
        let value = event.target.value;
        if (value.length <= 0) {
            value = undefined;
        }
        this.set(value);
        this.props.onUpdate.execute(value);
        this.validate();
    }
    validate() {
        let oldData = this.validatorData;
        this.validatorData = Validator.text(this.props.schema, this.get(), this.props.data, this.props.key);
        if (Utils.equals(oldData, this.validatorData) === false) {
            console.log("validate...");
            this.update();
        }
    }
    render() {
        let extra = CEUtils.parseInfo(this.validatorData);
        let input;
        if (this.props.schema.options !== undefined) {
            let values = this.props.schema.options;
            let current = this.get();
            let inputs = values.map(key => h("option", { value: key, selected: key === current }, key));
            input = h("select", { class: "InputContainer TextContainer Options", value: current, onInput: this.onInput }, inputs);
        }
        else if (this.props.schema.multiline === true) {
            input = h("textarea", { class: "InputContainer TextContainer Multiline", value: this.get(), onInput: this.onInput });
        }
        else {
            input = h("input", { class: "InputContainer TextContainer", type: "text", value: this.get(), onInput: this.onInput });
        }
        return h("div", { style: { "display": "inline-block" } },
            input,
            extra);
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], TextContainer.prototype, "onInput", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TextContainer.prototype, "validate", null);

var Type;
(function (Type) {
    // Null,
    Type["Unknown"] = "unknown";
    Type["Boolean"] = "boolean";
    Type["Number"] = "number";
    Type["String"] = "string";
    Type["Object"] = "object";
    Type["Array"] = "array";
    Type["Map"] = "map";
})(Type || (Type = {}));
var CEUtils;
(function (CEUtils) {
    function getNew(schema) {
        if (schema.type === Type.Object) {
            let out = {};
            for (const key in schema.properties) {
                if (schema.properties.hasOwnProperty(key)) {
                    const property = schema.properties[key];
                    out[key] = getNew(property);
                }
            }
            return out;
        }
        else if (schema.type === Type.Map) {
            return {};
        }
        else if (schema.type === Type.Array) {
            return [];
        }
        else if (schema.type === Type.Boolean) {
            if (schema.default !== undefined) {
                return schema.default;
            }
            if (schema.optional === true) {
                return undefined;
            }
            return false;
        }
        else if (schema.type === Type.Number) {
            if (schema.default !== undefined) {
                return schema.default;
            }
            if (schema.optional === true) {
                return undefined;
            }
            if (schema.min !== undefined) {
                return schema.min;
            }
            return 0;
        }
        else if (schema.type === Type.String) {
            if (schema.default !== undefined) {
                return schema.default;
            }
            if (schema.optional === true) {
                return undefined;
            }
            if (schema.options !== undefined && schema.options.length > 0) {
                return schema.options[0];
            }
            return "";
        }
        else {
            throw new Error("Invalid schema type");
        }
    }
    CEUtils.getNew = getNew;
    function renderInput(schema, data, key, onUpdate) {
        if (data === undefined) {
            throw new Error("Missing data");
        }
        if (schema.type === undefined) {
            throw new Error("Missing schema type");
        }
        if (schema.type === Type.Object) {
            return h(ObjectContainer, { schema: schema, data: data[key], onUpdate: onUpdate });
        }
        else if (schema.type === Type.Map) {
            return h(MapContainer, { schema: schema, data: data[key], onUpdate: onUpdate });
        }
        else if (schema.type === Type.Array) {
            return h(ArrayContainer, { schema: schema, data: data[key], onUpdate: onUpdate });
        }
        else if (schema.type === Type.Boolean) {
            return h(BooleanContainer, { schema: schema, data: data, key: key, onUpdate: onUpdate });
        }
        else if (schema.type === Type.Number) {
            return h(NumberContainer, { schema: schema, data: data, key: key, onUpdate: onUpdate });
        }
        else if (schema.type === Type.String) {
            return h(TextContainer, { schema: schema, data: data, key: key, onUpdate: onUpdate });
        }
        else {
            throw new Error("Invalid schema type");
        }
    }
    CEUtils.renderInput = renderInput;
    function parseInfo(data) {
        let extra = [];
        if (data.info.length > 0) {
            extra.push(h(TooltipProvider, { renderTooltip: () => h("div", { class: "TooltipContent" }, data.info.map(x => h("div", null, x))) },
                h("i", { class: "fas fa-info-circle", style: { "margin-left": 5, "color": "#88B" } })));
        }
        if (data.warnings.length > 0) {
            extra.push(h(TooltipProvider, { renderTooltip: () => h("div", { class: "TooltipContent" }, data.warnings.map(x => h("div", null, x))) },
                h("i", { class: "fas fa-exclamation-triangle", style: { "margin-left": 5, "color": "orange" } })));
        }
        if (data.errors.length > 0) {
            extra.push(h(TooltipProvider, { renderTooltip: () => h("div", { class: "TooltipContent" }, data.errors.map(x => h("div", null, x))) },
                h("i", { class: "far fa-times-circle", style: { "margin-left": 5, "color": "red" } })));
        }
        return extra;
    }
    CEUtils.parseInfo = parseInfo;
})(CEUtils || (CEUtils = {}));

class ElementContainer$2 extends BasicComponent {
    constructor() {
        super(...arguments);
        this.opened = true;
    }
    willReceiveProps() {
        this.update();
        this.initSchema();
    }
    initSchema() {
        let schema = this.props.properties;
        this.isBasic = false;
        if (schema.type === Type.Boolean || schema.type === Type.Number) {
            this.isBasic = true;
        }
        else if (schema.type === Type.String && schema.multiline !== true) {
            this.isBasic = true;
        }
    }
    delete() {
        this.props.data.splice(+this.props.key, 1);
        this.props.onDeleted();
        this.props.onUpdate.execute();
    }
    toggle() {
        this.opened = !this.opened;
        this.update();
    }
    render() {
        let { properties, data, key, onUpdate } = this.props;
        if (this.isBasic) {
            return h("div", { style: { "margin": "3px 0" } },
                h("i", { style: { "margin-right": 13 } }),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } }, key),
                h("i", { class: "far fa-trash-alt", style: { "color": "#C44", "margin-right": 5 }, onClick: this.delete }),
                CEUtils.renderInput(properties, data, key, onUpdate));
        }
        else if (this.opened) {
            return h("div", { style: { "margin": "3px 0" } },
                h("i", { class: "fas fa-caret-down", style: { "vertical-align": "top", "width": 10, "text-align": "center", "margin-right": 3 }, onClick: this.toggle }),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } }, key),
                h("i", { class: "far fa-trash-alt", style: { "color": "#C44" }, onClick: this.delete }),
                CEUtils.renderInput(properties, data, key, onUpdate));
        }
        else {
            return h("div", { style: { "margin": "3px 0" } },
                h("i", { class: "fas fa-caret-right", style: { "vertical-align": "top", "width": 10, "text-align": "center", "margin-right": 3 }, onClick: this.toggle }),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } }, key),
                h("i", { class: "far fa-trash-alt", style: { "color": "#C44" }, onClick: this.delete }));
        }
    }
}
__decorate([
    BasicComponent.initialize(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ElementContainer$2.prototype, "initSchema", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ElementContainer$2.prototype, "delete", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ElementContainer$2.prototype, "toggle", null);
class ArrayContainer extends BasicComponent {
    willReceiveProps() {
        this.update();
    }
    addNew() {
        this.props.data.push(CEUtils.getNew(this.props.schema.elements));
        this.update();
        this.props.onUpdate.execute();
    }
    onDeleted() {
        this.update();
    }
    render() {
        let { schema, data, onUpdate } = this.props;
        let inputs = [];
        let properties = schema.elements;
        for (const key in data) {
            if (data.hasOwnProperty(key)) {
                let i = +key;
                inputs.push(h(ElementContainer$2, { properties: properties, data: data, key: i, onUpdate: onUpdate, onDeleted: this.onDeleted }));
            }
        }
        return h("div", { style: { "padding-left": 10, "margin-left": 15 } },
            inputs,
            h("div", null,
                h("i", { class: "fas fa-plus-circle", style: { "color": "#4C4" }, onClick: this.addNew })));
    }
}
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ArrayContainer.prototype, "addNew", null);
__decorate([
    BasicComponent.bind(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ArrayContainer.prototype, "onDeleted", null);

class RootContainer extends BasicComponent {
    willReceiveProps() {
        this.update();
    }
    render() {
        let _a = this.props, { name, schema, data, onUpdate } = _a, props = __rest(_a, ["name", "schema", "data", "onUpdate"]);
        if (schema.type === Type.Object) {
            return h("div", Object.assign({}, props),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } },
                    name,
                    ": "),
                h(ObjectContainer, { schema: schema, data: data, onUpdate: onUpdate }));
        }
        else if (schema.type === Type.Map) {
            return h("div", Object.assign({}, props),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } },
                    name,
                    ": "),
                h(MapContainer, { schema: schema, data: data, onUpdate: onUpdate }));
        }
        else if (schema.type === Type.Array) {
            return h("div", Object.assign({}, props),
                h("span", { style: { "min-width": 150, "display": "inline-block", "vertical-align": "top" } },
                    name,
                    ": "),
                h(ArrayContainer, { schema: schema, data: data, onUpdate: onUpdate }));
        }
        throw new Error("Invalid schema type");
    }
}

// let ajv = new Ajv({});
// ajv.addMetaSchema(require("ajv/lib/refs/json-schema-draft-06.json"));
/*let Input = (element: JSX.Element) => {
    return {
        render: () => <span>AAA {element}</span>
    };
}*/
/*
boolean
number (int, float) (min, max)
string
text (multiline string)
array
map (js's object)
object

*/
const objectCombined = {
    type: Type.Object,
    properties: {
        object: {
            type: Type.Object,
            properties: {
                enabled: { type: Type.Boolean },
                id: { type: Type.Number, min: 1, max: 10 },
                name: { type: Type.String },
                description: { type: Type.String, multiline: true }
            }
        },
        map: {
            type: Type.Map,
            elements: {
                type: Type.Object,
                properties: {
                    enabled: { type: Type.Boolean },
                    id: { type: Type.Number, min: 1, max: 10 },
                    name: { type: Type.String },
                    description: { type: Type.String, multiline: true }
                }
            }
        },
        array: {
            type: Type.Array,
            elements: {
                type: Type.Object,
                properties: {
                    enabled: { type: Type.Boolean, default: true },
                    id: { type: Type.Number, min: 1, max: 10 },
                    name: { type: Type.String, default: "Default value" },
                    description: { type: Type.String, multiline: true, optional: true }
                }
            }
        },
        stringsArray: {
            type: Type.Array,
            elements: {
                type: Type.String,
                options: ["A", "B", "C", "D", "E", "F"]
            }
        },
        stringsMap: {
            type: Type.Map,
            elements: { type: Type.String }
        }
    }
};
const testSchema = objectCombined;
class ConfigEditor extends BasicComponent {
    constructor() {
        super(...arguments);
        this.onUpdate = new BasicEvent();
        this.data = {
            object: { enabled: true },
            map: {
                a: { name: "a" }, b: { id: 1 }, c: { name: "c" }
            },
            array: [{ name: "0" }, { id: 1 }, { name: "2" }],
            stringsArray: ["A", "", "C"],
            stringsMap: { a: "A", b: "1", c: "C" }
        };
    }
    initLink() {
        this.json = JSON.stringify(this.data, null, "\t");
        this.onUpdate.addCallback(() => {
            this.json = JSON.stringify(this.data, null, "\t");
        });
    }
    /*public validate() {
        let valid = ajv.validate(JSON.parse(this.schema), JSON.parse(this.json));
        if (!valid) {
            this.message = JSON.stringify(ajv.errors);
        } else {
            this.message = "valid";
        }
    }*/
    render() {
        return (h("div", { class: "JSONEditor" },
            h("div", { class: "RootContainer", style: { "flex": "1 0 0", margin: 10, overflow: "auto" } },
                h(RootContainer, { name: "Config", schema: testSchema, data: this.data, onUpdate: this.onUpdate, style: { "margin": "3px 0" } })),
            h("div", { class: "JSONContainer" },
                h(InputTextBox, { "$": this.$("json"), style: { "flex": "1 0 0", margin: 10 } })))
        /*<div>
            Schema: <InputTextBox $={this.$("schema")} /><br />
            JSON: <InputTextBox $={this.$("json")} /><br />
            <BasicText $={this.$("message")} />
        </div>*/
        /*

                {renderContainer(testSchema, "root", this.data, this.onUpdate)}<br /><br />
        */
        );
    }
}
__decorate([
    BasicComponent.var(),
    __metadata("design:type", String)
], ConfigEditor.prototype, "json", void 0);
__decorate([
    BasicComponent.initialize(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ConfigEditor.prototype, "initLink", null);

require("./style.css");

export { options, VNode, h, renderClass, render, $base, TSX, Component, Overlay, BasicComponent, BasicInput, InputText, InputNumber, InputRange, InputTextBox, InputCheckbox, BasicText, FixedNumber, InputSelect, InputFile, InputFiles, JSONView, Tooltip, TooltipProvider, ListContainer, SelectContainer, ListSelect, Fragment, ArrayContainer, BooleanContainer, MapContainer, NumberContainer, ObjectContainer, RootContainer, TextContainer, ConfigEditor, Type, CEUtils, ValidatorType, Validator };

//# sourceMappingURL=index.js.map