declare module "TSX/TSX" {
    import { BaseClass } from "@h4x/common";
    export const options: {
        debug?: boolean;
        verbose?: boolean;
    };
    export class VNode<P = any> {
        type: (new () => Component<P>) | string;
        children: ComponentChild[];
        props: P;
        key?: string | number;
    }
    export type ComponentChild = VNode | string | number | null;
    export type ComponentChildren = ComponentChild[] | ComponentChild;
    export interface Attributes<T> {
        key?: string | number | any;
        ref?: (instance: T) => void;
    }
    export type ComponentProps<P, RefType = any> = Readonly<Attributes<RefType> & P>;
    export function h<P>(node: (new () => Component<P>), props: Attributes<any> & P | null, ...children: ComponentChildren[]): VNode<P>;
    export function h(node: string, props: JSX.HTMLAttributes<any> & JSX.SVGAttributes & Record<string, any> | null, ...children: ComponentChildren[]): VNode;
    export function renderClass(Class: (new () => Component<any>), parent: Element | Document, before?: Node | number): Element | undefined;
    export function render(vnode: ComponentChild, parent: Element | Document, before?: Node | number): Element | undefined;
    export const $base: unique symbol;
    const $updateProps: unique symbol;
    const $update: unique symbol;
    const $init: unique symbol;
    const $attached: unique symbol;
    const $detached: unique symbol;
    const $destroy: unique symbol;
    const $class: unique symbol;
    export namespace TSX {
        export function configureInterval(fps: number): void;
        export type Props<T> = T extends Component<infer R> ? R : unknown;
        export class Rendered<T extends VComponent<any> | VDomNode = VComponent<any> | VDomNode> {
            readonly vnode: T;
            readonly element: Element;
            private __attached;
            get attached(): boolean;
            constructor(vnode: T, element: Element);
            attach(parent: Element | Document, before?: Node | number): void;
            detach(): void;
            /** @deprecated Internal */
            destroy(): void;
        }
        export type RenderedComponent<T extends Component<any>> = Rendered<VComponent<T>>;
        export type RenderedElement = Rendered<VDomElement | VComponent<any>>;
        export function prerender(vnode: VNode<any>): RenderedElement;
        export function prerender(vnode: string | number | null): Rendered<VDomNode>;
        export function prerender(vnode: ComponentChild): Rendered;
        export function prerenderComponent<T extends Component<any>>(vnode: ComponentChild): RenderedComponent<T>;
        export class Renderer<T extends (...args: any) => JSX.Element = () => JSX.Element> {
            private component;
            private fn;
            private args;
            private __rendered;
            get rendered(): boolean;
            readonly vnode: VComponent<any> | VDomNode;
            readonly element: Element;
            private constructor();
            update(): void;
            forceInstantUpdate(): void;
            private _persistent;
            get persistent(): boolean;
            private render;
            [$update](): void;
            /** @Internal */
            static renderNode(renderer: Renderer, persistent: boolean): any;
            /** @Internal */
            static attachNode(renderer: Renderer, vdom: VDomElement | VComponent<Component<any>>): void;
            /** @Internal */
            static detachNode(renderer: Renderer): void;
            static create<T extends (...args: any) => JSX.Element>(component: Component<any>, fn: T, args: FunctionArgs<T>): Renderer;
        }
        abstract class VDom {
            readonly key?: string | number;
            isActive: boolean;
            renderer?: Renderer;
            isCompatible(vnode: ComponentChild): boolean;
            abstract update(vnode: ComponentChild): void;
            attachRenderer(renderer: Renderer): void;
        }
        abstract class VDomNode extends VDom {
            protected nodeName: string;
            node: Node;
            children: (VDomNode | VComponent<any>)[];
            mappedChildren: Map<string | number, VComponent<any> | VDomNode>;
            inlined: Set<Renderer<() => JSX.Element>>;
            isPersistent: boolean;
            get typeName(): string;
            constructor();
            init(node: Node, key: string | number | undefined): this;
            abstract destroy(): void;
            addChild(vdom: VDomNode | VComponent<any>): void;
            updateChildren(vnode: VNode): void;
            setText(text: string): void;
        }
        class VDomElement extends VDomNode {
            readonly node: Element;
            private [$class];
            private props;
            destroy(): void;
            get typeName(): string;
            applyAttributes(target: any): any;
            private listeners;
            get eventHandler(): (this: Node, e: Event) => void;
            private applyAttribute;
            update(vnode: VNode): void;
            isCompatible(vnode: ComponentChild): boolean;
            setText(text: string): void;
        }
        export class VComponent<T extends Component<any>> extends VDom {
            instance: T;
            vdom: VDomElement | VComponent<any>;
            isPersistent: boolean;
            get node(): Node;
            constructor(type: new () => T, props: any, children: ComponentChild[]);
            destroy(): void;
            onAttached(): void;
            onDetached(): void;
            get typeName(): string;
            componentUpdate(vnode: VNode): void;
            update(vnode: VNode): void;
            isCompatible(vnode: ComponentChild): boolean;
        }
        export {};
    }
    type FunctionArgs<T extends (...args: any) => any> = T extends (...args: infer P) => any ? P : never;
    export abstract class Component<P> extends BaseClass {
        static readonly Children: unique symbol;
        private static _REF;
        readonly REF: string;
        [$base]: TSX.VComponent<this>;
        private updateScheduled;
        private internalUpdating;
        private internalDestroyed;
        props: ComponentProps<P, this>;
        children: ComponentChild[];
        constructor();
        protected createRenderer<T extends (...args: any) => JSX.Element>(fn: T, ...args: FunctionArgs<T>): TSX.Renderer;
        abstract render(): VNode<this>;
        [$init](props: ComponentProps<P, any>, children: ComponentChild[]): void;
        protected init(): void;
        [$destroy](): void;
        protected destroy(): void;
        onAttached(): void;
        onDetached(): void;
        [$attached](): void;
        [$detached](): void;
        [$update](): void;
        protected update(): void;
        protected forceInstantUpdate(): void;
        protected preUpdate(): void;
        protected postUpdate(): void;
        protected receivedProps(props: ComponentProps<P, this>, children: ComponentChild[]): void;
        protected willReceiveProps(props: ComponentProps<P, this>, children: ComponentChild[]): void;
        [$updateProps](props: ComponentProps<P, this>, children: ComponentChild[]): void;
    }
    export namespace Component { }
}
declare module "Components/BasicComponent" {
    import { Component } from "TSX/TSX";
    import { BasicEvent, EventInterface } from "@h4x/common";
    export type $Proxy<T> = {
        get: () => T;
        set: (variable: T) => void;
        onUpdate: BasicEvent<(value?: T) => void>;
        onChange: BasicEvent<(value?: T) => void>;
    };
    export abstract class BasicComponent<P = {}> extends Component<P> {
        private attachedEventReferences;
        protected onDestroy: BasicEvent<() => void, () => void>;
        constructor();
        proxy<K extends keyof this = keyof this>(key: K): $Proxy<this[K]>;
        $<K extends keyof this = keyof this>(key: K): $Proxy<this[K]>;
        static var(): (target: any, key: string) => void;
        static bind<T extends BasicComponent<any>>(): (target: T, key: string) => void;
        static interval<T extends BasicComponent<any>>(time: number): (target: T, key: string) => void;
        static initialize<T extends BasicComponent<any>>(): (target: T, key: string) => void;
        private __init;
        private __destroy;
        /** Attach event reference to be cleaned up when the object is destroyed */
        protected ref<T extends EventInterface<any, any>>(ref: T): T;
        static normalizeClass(target: any): undefined | {
            [K: string]: boolean;
        };
        static mergeClasses(...target: (string | {
            [K: string]: boolean | undefined;
        } | (string | false | undefined | null)[] | undefined)[]): undefined | {
            [K: string]: boolean;
        };
    }
}
declare module "TSX/Overlay" {
    import { BasicComponent } from "Components/BasicComponent";
    import { VNode } from "TSX/TSX";
    import "./Overlay.scss";
    export class Overlay extends BasicComponent<{}> {
        private queue;
        private alerts;
        private floating;
        private static instance;
        constructor();
        static create(element: HTMLElement): any;
        private static uniqueID;
        static add(vnode: VNode): {
            vnode: VNode<any>;
            instance: BasicComponent<{}>;
        };
        static remove(component: BasicComponent): any;
        static addAlert(vnode: VNode): {
            vnode: VNode<any>;
            instance: BasicComponent<{}>;
        };
        static removeAlert(component: BasicComponent): any;
        static addFloating(vnode: VNode): {
            vnode: VNode<any>;
            instance: BasicComponent<{}>;
        };
        static removeFloating(component: BasicComponent): any;
        postUpdate(): void;
        render(): JSX.Element;
        private static createInstance;
        private static removeInstance;
    }
}
declare module "TSX/global" {
    import { Component, VNode, ComponentChildren, Attributes, TSX } from "TSX/TSX";
    type DomElement = Element;
    type DomSVGElement = SVGElement;
    global {
        namespace Dom {
            type Element = DomElement;
            type SVGElement = DomSVGElement;
        }
    }
    global {
        namespace JSX {
            interface Element extends VNode {
            }
            interface ElementClass extends Component<any> {
            }
            interface ElementAttributesProperty {
                props: any;
            }
            interface ElementChildrenAttribute {
                [Component.Children]: any;
            }
            interface SVGAttributes<T extends Dom.SVGElement = Dom.SVGElement> extends HTMLAttributes<T> {
                accentHeight?: number | string;
                accumulate?: "none" | "sum";
                additive?: "replace" | "sum";
                alignmentBaseline?: "auto" | "baseline" | "before-edge" | "text-before-edge" | "middle" | "central" | "after-edge" | "text-after-edge" | "ideographic" | "alphabetic" | "hanging" | "mathematical" | "inherit";
                allowReorder?: "no" | "yes";
                alphabetic?: number | string;
                amplitude?: number | string;
                arabicForm?: "initial" | "medial" | "terminal" | "isolated";
                ascent?: number | string;
                attributeName?: string;
                attributeType?: string;
                autoReverse?: number | string;
                azimuth?: number | string;
                baseFrequency?: number | string;
                baselineShift?: number | string;
                baseProfile?: number | string;
                bbox?: number | string;
                begin?: number | string;
                bias?: number | string;
                by?: number | string;
                calcMode?: number | string;
                capHeight?: number | string;
                clip?: number | string;
                clipPath?: string;
                clipPathUnits?: number | string;
                clipRule?: number | string;
                colorInterpolation?: number | string;
                colorInterpolationFilters?: "auto" | "sRGB" | "linearRGB" | "inherit";
                colorProfile?: number | string;
                colorRendering?: number | string;
                contentScriptType?: number | string;
                contentStyleType?: number | string;
                cursor?: number | string;
                cx?: number | string;
                cy?: number | string;
                d?: string;
                decelerate?: number | string;
                descent?: number | string;
                diffuseConstant?: number | string;
                direction?: number | string;
                display?: number | string;
                divisor?: number | string;
                dominantBaseline?: number | string;
                dur?: number | string;
                dx?: number | string;
                dy?: number | string;
                edgeMode?: number | string;
                elevation?: number | string;
                enableBackground?: number | string;
                end?: number | string;
                exponent?: number | string;
                externalResourcesRequired?: number | string;
                fill?: string;
                fillOpacity?: number | string;
                fillRule?: "nonzero" | "evenodd" | "inherit";
                filter?: string;
                filterRes?: number | string;
                filterUnits?: number | string;
                floodColor?: number | string;
                floodOpacity?: number | string;
                focusable?: number | string;
                fontFamily?: string;
                fontSize?: number | string;
                fontSizeAdjust?: number | string;
                fontStretch?: number | string;
                fontStyle?: number | string;
                fontVariant?: number | string;
                fontWeight?: number | string;
                format?: number | string;
                from?: number | string;
                fx?: number | string;
                fy?: number | string;
                g1?: number | string;
                g2?: number | string;
                glyphName?: number | string;
                glyphOrientationHorizontal?: number | string;
                glyphOrientationVertical?: number | string;
                glyphRef?: number | string;
                gradientTransform?: string;
                gradientUnits?: string;
                hanging?: number | string;
                horizAdvX?: number | string;
                horizOriginX?: number | string;
                ideographic?: number | string;
                imageRendering?: number | string;
                in2?: number | string;
                in?: string;
                intercept?: number | string;
                k1?: number | string;
                k2?: number | string;
                k3?: number | string;
                k4?: number | string;
                k?: number | string;
                kernelMatrix?: number | string;
                kernelUnitLength?: number | string;
                kerning?: number | string;
                keyPoints?: number | string;
                keySplines?: number | string;
                keyTimes?: number | string;
                lengthAdjust?: number | string;
                letterSpacing?: number | string;
                lightingColor?: number | string;
                limitingConeAngle?: number | string;
                local?: number | string;
                markerEnd?: string;
                markerHeight?: number | string;
                markerMid?: string;
                markerStart?: string;
                markerUnits?: number | string;
                markerWidth?: number | string;
                mask?: string;
                maskContentUnits?: number | string;
                maskUnits?: number | string;
                mathematical?: number | string;
                mode?: number | string;
                numOctaves?: number | string;
                offset?: number | string;
                opacity?: number | string;
                operator?: number | string;
                order?: number | string;
                orient?: number | string;
                orientation?: number | string;
                origin?: number | string;
                overflow?: number | string;
                overlinePosition?: number | string;
                overlineThickness?: number | string;
                paintOrder?: number | string;
                panose1?: number | string;
                pathLength?: number | string;
                patternContentUnits?: string;
                patternTransform?: number | string;
                patternUnits?: string;
                pointerEvents?: number | string;
                points?: string;
                pointsAtX?: number | string;
                pointsAtY?: number | string;
                pointsAtZ?: number | string;
                preserveAlpha?: number | string;
                preserveAspectRatio?: string;
                primitiveUnits?: number | string;
                r?: number | string;
                radius?: number | string;
                refX?: number | string;
                refY?: number | string;
                renderingIntent?: number | string;
                repeatCount?: number | string;
                repeatDur?: number | string;
                requiredExtensions?: number | string;
                requiredFeatures?: number | string;
                restart?: number | string;
                result?: string;
                rotate?: number | string;
                rx?: number | string;
                ry?: number | string;
                scale?: number | string;
                seed?: number | string;
                shapeRendering?: number | string;
                slope?: number | string;
                spacing?: number | string;
                specularConstant?: number | string;
                specularExponent?: number | string;
                speed?: number | string;
                spreadMethod?: string;
                startOffset?: number | string;
                stdDeviation?: number | string;
                stemh?: number | string;
                stemv?: number | string;
                stitchTiles?: number | string;
                stopColor?: string;
                stopOpacity?: number | string;
                strikethroughPosition?: number | string;
                strikethroughThickness?: number | string;
                string?: number | string;
                stroke?: string;
                strokeDasharray?: string | number;
                strokeDashoffset?: string | number;
                strokeLinecap?: "butt" | "round" | "square" | "inherit";
                strokeLinejoin?: "miter" | "round" | "bevel" | "inherit";
                strokeMiterlimit?: string;
                strokeOpacity?: number | string;
                strokeWidth?: number | string;
                surfaceScale?: number | string;
                systemLanguage?: number | string;
                tableValues?: number | string;
                targetX?: number | string;
                targetY?: number | string;
                textAnchor?: string;
                textDecoration?: number | string;
                textLength?: number | string;
                textRendering?: number | string;
                to?: number | string;
                transform?: string;
                u1?: number | string;
                u2?: number | string;
                underlinePosition?: number | string;
                underlineThickness?: number | string;
                unicode?: number | string;
                unicodeBidi?: number | string;
                unicodeRange?: number | string;
                unitsPerEm?: number | string;
                vAlphabetic?: number | string;
                values?: string;
                vectorEffect?: number | string;
                version?: string;
                vertAdvY?: number | string;
                vertOriginX?: number | string;
                vertOriginY?: number | string;
                vHanging?: number | string;
                vIdeographic?: number | string;
                viewBox?: string;
                viewTarget?: number | string;
                visibility?: number | string;
                vMathematical?: number | string;
                widths?: number | string;
                wordSpacing?: number | string;
                writingMode?: number | string;
                x1?: number | string;
                x2?: number | string;
                x?: number | string;
                xChannelSelector?: string;
                xHeight?: number | string;
                xlinkActuate?: string;
                xlinkArcrole?: string;
                xlinkHref?: string;
                xlinkRole?: string;
                xlinkShow?: string;
                xlinkTitle?: string;
                xlinkType?: string;
                xmlBase?: string;
                xmlLang?: string;
                xmlns?: string;
                xmlnsXlink?: string;
                xmlSpace?: string;
                y1?: number | string;
                y2?: number | string;
                y?: number | string;
                yChannelSelector?: string;
                z?: number | string;
                zoomAndPan?: string;
            }
            interface PathAttributes {
                d: string;
            }
            interface EventHandler<E extends Event, T extends Dom.Element> {
                (event: E, element: T): void;
            }
            type ClipboardEventHandler<T extends Dom.Element> = EventHandler<ClipboardEvent, T>;
            type CompositionEventHandler<T extends Dom.Element> = EventHandler<CompositionEvent, T>;
            type DragEventHandler<T extends Dom.Element> = EventHandler<DragEvent, T>;
            type FocusEventHandler<T extends Dom.Element> = EventHandler<FocusEvent, T>;
            type KeyboardEventHandler<T extends Dom.Element> = EventHandler<KeyboardEvent, T>;
            type MouseEventHandler<T extends Dom.Element> = EventHandler<MouseEvent, T>;
            type TouchEventHandler<T extends Dom.Element> = EventHandler<TouchEvent, T>;
            type UIEventHandler<T extends Dom.Element> = EventHandler<UIEvent, T>;
            type WheelEventHandler<T extends Dom.Element> = EventHandler<WheelEvent, T>;
            type AnimationEventHandler<T extends Dom.Element> = EventHandler<AnimationEvent, T>;
            type TransitionEventHandler<T extends Dom.Element> = EventHandler<TransitionEvent, T>;
            type GenericEventHandler<T extends Dom.Element> = EventHandler<Event, T>;
            interface DOMAttributes<T extends Dom.Element> {
                [Component.Children]?: ComponentChildren | object;
                dangerouslySetInnerHTML?: {
                    __html: string;
                };
                onLoad?: GenericEventHandler<T>;
                onCopy?: ClipboardEventHandler<T>;
                onCut?: ClipboardEventHandler<T>;
                onPaste?: ClipboardEventHandler<T>;
                onCompositionEnd?: CompositionEventHandler<T>;
                onCompositionStart?: CompositionEventHandler<T>;
                onCompositionUpdate?: CompositionEventHandler<T>;
                onFocus?: FocusEventHandler<T>;
                onBlur?: FocusEventHandler<T>;
                onChange?: GenericEventHandler<T>;
                onInput?: GenericEventHandler<T>;
                onSearch?: GenericEventHandler<T>;
                onSubmit?: GenericEventHandler<T>;
                onKeyDown?: KeyboardEventHandler<T>;
                onKeyPress?: KeyboardEventHandler<T>;
                onKeyUp?: KeyboardEventHandler<T>;
                onAbort?: GenericEventHandler<T>;
                onCanPlay?: GenericEventHandler<T>;
                onCanPlayThrough?: GenericEventHandler<T>;
                onDurationChange?: GenericEventHandler<T>;
                onEmptied?: GenericEventHandler<T>;
                onEncrypted?: GenericEventHandler<T>;
                onEnded?: GenericEventHandler<T>;
                onLoadedData?: GenericEventHandler<T>;
                onLoadedMetadata?: GenericEventHandler<T>;
                onLoadStart?: GenericEventHandler<T>;
                onPause?: GenericEventHandler<T>;
                onPlay?: GenericEventHandler<T>;
                onPlaying?: GenericEventHandler<T>;
                onProgress?: GenericEventHandler<T>;
                onRateChange?: GenericEventHandler<T>;
                onSeeked?: GenericEventHandler<T>;
                onSeeking?: GenericEventHandler<T>;
                onStalled?: GenericEventHandler<T>;
                onSuspend?: GenericEventHandler<T>;
                onTimeUpdate?: GenericEventHandler<T>;
                onVolumeChange?: GenericEventHandler<T>;
                onWaiting?: GenericEventHandler<T>;
                onClick?: MouseEventHandler<T>;
                onContextMenu?: MouseEventHandler<T>;
                onDblClick?: MouseEventHandler<T>;
                onDrag?: DragEventHandler<T>;
                onDragEnd?: DragEventHandler<T>;
                onDragEnter?: DragEventHandler<T>;
                onDragExit?: DragEventHandler<T>;
                onDragLeave?: DragEventHandler<T>;
                onDragOver?: DragEventHandler<T>;
                onDragStart?: DragEventHandler<T>;
                onDrop?: DragEventHandler<T>;
                onMouseDown?: MouseEventHandler<T>;
                onMouseEnter?: MouseEventHandler<T>;
                onMouseLeave?: MouseEventHandler<T>;
                onMouseMove?: MouseEventHandler<T>;
                onMouseOut?: MouseEventHandler<T>;
                onMouseOver?: MouseEventHandler<T>;
                onMouseUp?: MouseEventHandler<T>;
                onSelect?: GenericEventHandler<T>;
                onTouchCancel?: TouchEventHandler<T>;
                onTouchEnd?: TouchEventHandler<T>;
                onTouchMove?: TouchEventHandler<T>;
                onTouchStart?: TouchEventHandler<T>;
                onScroll?: UIEventHandler<T>;
                onWheel?: WheelEventHandler<T>;
                onAnimationStart?: AnimationEventHandler<T>;
                onAnimationEnd?: AnimationEventHandler<T>;
                onAnimationIteration?: AnimationEventHandler<T>;
                onTransitionEnd?: TransitionEventHandler<T>;
            }
            interface HTMLAttributes<T extends Dom.Element = Dom.Element> extends Attributes<any>, DOMAttributes<T> {
                accept?: string;
                acceptCharset?: string;
                accessKey?: string;
                action?: string;
                allowFullScreen?: boolean;
                allowTransparency?: boolean;
                alt?: string;
                async?: boolean;
                autocomplete?: string;
                autofocus?: boolean;
                autoPlay?: boolean;
                capture?: boolean;
                cellPadding?: number | string;
                cellSpacing?: number | string;
                charSet?: string;
                challenge?: string;
                checked?: boolean;
                class?: string | {
                    [K: string]: boolean | undefined;
                } | (string | false | undefined | null)[];
                className?: string | {
                    [K: string]: boolean | undefined;
                } | (string | false | undefined | null)[];
                cols?: number;
                colSpan?: number;
                content?: string;
                contentEditable?: boolean;
                contextMenu?: string;
                controls?: boolean;
                coords?: string;
                crossOrigin?: string;
                data?: string;
                dateTime?: string;
                default?: boolean;
                defer?: boolean;
                dir?: string;
                disabled?: boolean;
                download?: any;
                draggable?: boolean;
                encType?: string;
                form?: string;
                formAction?: string;
                formEncType?: string;
                formMethod?: string;
                formNoValidate?: boolean;
                formTarget?: string;
                frameBorder?: number | string;
                headers?: string;
                height?: number | string;
                hidden?: boolean;
                high?: number;
                href?: string;
                hrefLang?: string;
                for?: string;
                httpEquiv?: string;
                icon?: string;
                id?: string;
                inputMode?: string;
                integrity?: string;
                is?: string;
                keyParams?: string;
                keyType?: string;
                kind?: string;
                label?: string;
                lang?: string;
                list?: string;
                loop?: boolean;
                low?: number;
                manifest?: string;
                marginHeight?: number;
                marginWidth?: number;
                max?: number | string;
                maxLength?: number;
                media?: string;
                mediaGroup?: string;
                method?: string;
                min?: number | string;
                minLength?: number;
                multiple?: boolean;
                muted?: boolean;
                name?: string;
                noValidate?: boolean;
                open?: boolean;
                optimum?: number;
                pattern?: string;
                placeholder?: string;
                poster?: string;
                preload?: string;
                radioGroup?: string;
                readOnly?: boolean;
                rel?: string;
                required?: boolean;
                role?: string;
                rows?: number;
                rowSpan?: number;
                sandbox?: string;
                scope?: string;
                scoped?: boolean;
                scrolling?: string;
                seamless?: boolean;
                selected?: boolean;
                shape?: string;
                size?: number;
                sizes?: string;
                slot?: string;
                span?: number;
                spellCheck?: boolean;
                src?: string;
                srcset?: string;
                srcDoc?: string;
                srcLang?: string;
                srcSet?: string;
                start?: number;
                step?: number | string;
                style?: any;
                summary?: string;
                tabIndex?: number;
                target?: string;
                title?: string;
                type?: string;
                useMap?: string;
                value?: string | string[] | number;
                width?: number | string;
                wmode?: string;
                wrap?: string;
                about?: string;
                datatype?: string;
                inlist?: any;
                prefix?: string;
                property?: string;
                resource?: string;
                typeof?: string;
                vocab?: string;
                raw?: boolean;
            }
            interface IntrinsicElements {
                a: HTMLAttributes<HTMLAnchorElement>;
                abbr: HTMLAttributes;
                address: HTMLAttributes;
                area: HTMLAttributes<HTMLAreaElement>;
                article: HTMLAttributes;
                aside: HTMLAttributes;
                audio: HTMLAttributes<HTMLAudioElement>;
                b: HTMLAttributes;
                base: HTMLAttributes<HTMLBaseElement>;
                bdi: HTMLAttributes;
                bdo: HTMLAttributes;
                big: HTMLAttributes;
                blockquote: HTMLAttributes<HTMLQuoteElement>;
                body: HTMLAttributes<HTMLBodyElement>;
                br: HTMLAttributes<HTMLBRElement>;
                button: HTMLAttributes<HTMLButtonElement>;
                canvas: HTMLAttributes<HTMLCanvasElement>;
                caption: HTMLAttributes<HTMLTableCaptionElement>;
                cite: HTMLAttributes;
                code: HTMLAttributes;
                col: HTMLAttributes<HTMLTableColElement>;
                colgroup: HTMLAttributes<HTMLTableColElement>;
                data: HTMLAttributes<HTMLDataElement>;
                datalist: HTMLAttributes<HTMLDataListElement>;
                dd: HTMLAttributes;
                del: HTMLAttributes<HTMLModElement>;
                details: HTMLAttributes<HTMLDetailsElement>;
                dfn: HTMLAttributes;
                div: HTMLAttributes<HTMLDivElement>;
                dl: HTMLAttributes<HTMLDListElement>;
                dt: HTMLAttributes;
                em: HTMLAttributes;
                embed: HTMLAttributes<HTMLEmbedElement>;
                fieldset: HTMLAttributes<HTMLFieldSetElement>;
                figcaption: HTMLAttributes;
                figure: HTMLAttributes;
                footer: HTMLAttributes;
                form: HTMLAttributes<HTMLFormElement>;
                h1: HTMLAttributes<HTMLHeadingElement>;
                h2: HTMLAttributes<HTMLHeadingElement>;
                h3: HTMLAttributes<HTMLHeadingElement>;
                h4: HTMLAttributes<HTMLHeadingElement>;
                h5: HTMLAttributes<HTMLHeadingElement>;
                h6: HTMLAttributes<HTMLHeadingElement>;
                head: HTMLAttributes<HTMLHeadElement>;
                header: HTMLAttributes;
                hr: HTMLAttributes<HTMLHRElement>;
                html: HTMLAttributes<HTMLHtmlElement>;
                i: HTMLAttributes;
                iframe: HTMLAttributes<HTMLIFrameElement>;
                img: HTMLAttributes<HTMLImageElement>;
                input: HTMLAttributes<HTMLInputElement>;
                ins: HTMLAttributes<HTMLModElement>;
                kbd: HTMLAttributes;
                keygen: HTMLAttributes;
                label: HTMLAttributes<HTMLLabelElement>;
                legend: HTMLAttributes<HTMLLegendElement>;
                li: HTMLAttributes<HTMLLIElement>;
                link: HTMLAttributes<HTMLLinkElement>;
                main: HTMLAttributes;
                map: HTMLAttributes<HTMLMapElement>;
                mark: HTMLAttributes;
                menu: HTMLAttributes<HTMLMenuElement>;
                menuitem: HTMLAttributes;
                meta: HTMLAttributes<HTMLMetaElement>;
                meter: HTMLAttributes<HTMLMeterElement>;
                nav: HTMLAttributes;
                noscript: HTMLAttributes;
                object: HTMLAttributes<HTMLObjectElement>;
                ol: HTMLAttributes<HTMLOListElement>;
                optgroup: HTMLAttributes<HTMLOptGroupElement>;
                option: HTMLAttributes<HTMLOptionElement>;
                output: HTMLAttributes<HTMLOutputElement>;
                p: HTMLAttributes<HTMLParagraphElement>;
                param: HTMLAttributes<HTMLParamElement>;
                picture: HTMLAttributes<HTMLPictureElement>;
                pre: HTMLAttributes<HTMLPreElement>;
                progress: HTMLAttributes<HTMLProgressElement>;
                q: HTMLAttributes<HTMLQuoteElement>;
                rp: HTMLAttributes;
                rt: HTMLAttributes;
                ruby: HTMLAttributes;
                s: HTMLAttributes;
                samp: HTMLAttributes;
                script: HTMLAttributes<HTMLScriptElement>;
                section: HTMLAttributes;
                select: HTMLAttributes<HTMLSelectElement>;
                slot: HTMLAttributes<HTMLSlotElement>;
                small: HTMLAttributes;
                source: HTMLAttributes<HTMLSourceElement>;
                span: HTMLAttributes<HTMLSpanElement>;
                strong: HTMLAttributes;
                style: HTMLAttributes<HTMLStyleElement>;
                sub: HTMLAttributes;
                summary: HTMLAttributes;
                sup: HTMLAttributes;
                table: HTMLAttributes<HTMLTableElement>;
                tbody: HTMLAttributes<HTMLTableSectionElement>;
                td: HTMLAttributes<HTMLTableCellElement>;
                textarea: HTMLAttributes<HTMLTextAreaElement>;
                tfoot: HTMLAttributes<HTMLTableSectionElement>;
                th: HTMLAttributes<HTMLTableCellElement>;
                thead: HTMLAttributes<HTMLTableSectionElement>;
                time: HTMLAttributes<HTMLTimeElement>;
                title: HTMLAttributes<HTMLTitleElement>;
                tr: HTMLAttributes<HTMLTableRowElement>;
                track: HTMLAttributes<HTMLTrackElement>;
                u: HTMLAttributes;
                ul: HTMLAttributes<HTMLUListElement>;
                var: HTMLAttributes;
                video: HTMLAttributes<HTMLVideoElement>;
                wbr: HTMLAttributes;
                animate: SVGAttributes<SVGAnimateElement>;
                animateMotion: SVGAttributes<SVGAnimateMotionElement>;
                animateTransform: SVGAttributes<SVGAnimateTransformElement>;
                circle: SVGAttributes<SVGCircleElement>;
                clipPath: SVGAttributes<SVGClipPathElement>;
                defs: SVGAttributes<SVGDefsElement>;
                desc: SVGAttributes<SVGDescElement>;
                ellipse: SVGAttributes<SVGEllipseElement>;
                feBlend: SVGAttributes<SVGFEBlendElement>;
                feColorMatrix: SVGAttributes<SVGFEColorMatrixElement>;
                feComponentTransfer: SVGAttributes<SVGFEComponentTransferElement>;
                feComposite: SVGAttributes<SVGFECompositeElement>;
                feConvolveMatrix: SVGAttributes<SVGFEConvolveMatrixElement>;
                feDiffuseLighting: SVGAttributes<SVGFEDiffuseLightingElement>;
                feDisplacementMap: SVGAttributes<SVGFEDisplacementMapElement>;
                feDistantLight: SVGAttributes<SVGFEDistantLightElement>;
                feDropShadow: SVGAttributes<SVGFEDropShadowElement>;
                feFlood: SVGAttributes<SVGFEFloodElement>;
                feFuncA: SVGAttributes<SVGFEFuncAElement>;
                feFuncB: SVGAttributes<SVGFEFuncBElement>;
                feFuncG: SVGAttributes<SVGFEFuncGElement>;
                feFuncR: SVGAttributes<SVGFEFuncRElement>;
                feGaussianBlur: SVGAttributes<SVGFEGaussianBlurElement>;
                feImage: SVGAttributes<SVGFEImageElement>;
                feMerge: SVGAttributes<SVGFEMergeElement>;
                feMergeNode: SVGAttributes<SVGFEMergeNodeElement>;
                feMorphology: SVGAttributes<SVGFEMorphologyElement>;
                feOffset: SVGAttributes<SVGFEOffsetElement>;
                fePointLight: SVGAttributes<SVGFEPointLightElement>;
                feSpecularLighting: SVGAttributes<SVGFESpecularLightingElement>;
                feSpotLight: SVGAttributes<SVGFESpotLightElement>;
                feTile: SVGAttributes<SVGFETileElement>;
                feTurbulence: SVGAttributes<SVGFETurbulenceElement>;
                filter: SVGAttributes<SVGFilterElement>;
                foreignObject: SVGAttributes<SVGForeignObjectElement>;
                g: SVGAttributes<SVGGElement>;
                image: SVGAttributes<SVGImageElement>;
                line: SVGAttributes<SVGLineElement>;
                linearGradient: SVGAttributes<SVGLinearGradientElement>;
                marker: SVGAttributes<SVGMarkerElement>;
                mask: SVGAttributes<SVGMaskElement>;
                metadata: SVGAttributes<SVGMetadataElement>;
                mpath: SVGAttributes<SVGMPathElement>;
                path: SVGAttributes<SVGPathElement>;
                pattern: SVGAttributes<SVGPatternElement>;
                polygon: SVGAttributes<SVGPolygonElement>;
                polyline: SVGAttributes<SVGPolylineElement>;
                radialGradient: SVGAttributes<SVGRadialGradientElement>;
                rect: SVGAttributes<SVGRectElement>;
                set: SVGAttributes<SVGSetElement>;
                stop: SVGAttributes<SVGStopElement>;
                svg: SVGAttributes<SVGSVGElement>;
                switch: SVGAttributes<SVGSwitchElement>;
                symbol: SVGAttributes<SVGSymbolElement>;
                text: SVGAttributes<SVGTextElement>;
                textPath: SVGAttributes<SVGTextPathElement>;
                tspan: SVGAttributes<SVGTSpanElement>;
                use: SVGAttributes<SVGUseElement>;
                view: SVGAttributes<SVGViewElement>;
                inline: {
                    renderer: TSX.Renderer;
                    persistent?: boolean;
                    [Component.Children]?: undefined;
                };
                /** Raw elements require key, ref and can not specify children using TSX */
                raw: HTMLAttributes<HTMLDivElement> & {
                    key: string | number;
                    ref: (instance: HTMLDivElement) => void;
                    [Component.Children]?: undefined;
                };
            }
        }
    }
}
declare module "Components/FormComponents" {
    import { $Proxy, BasicComponent } from "Components/BasicComponent";
    export interface InputProps<T> extends JSX.HTMLAttributes<HTMLInputElement> {
        $: $Proxy<T>;
        value?: undefined;
        onInput?: undefined;
        onChange?: undefined;
    }
    export abstract class BasicInput<T extends {
        $: $Proxy<any>;
    }> extends BasicComponent<T> {
        init(): void;
        destroy(): void;
        willReceiveProps(): void;
    }
    export class InputText extends BasicInput<InputProps<string>> {
        protected onInput(event: any): void;
        protected onChange(event: any): void;
        render(): JSX.Element;
    }
    export class InputNumber extends BasicInput<InputProps<number>> {
        protected onInput(event: any): void;
        protected onChange(event: any): void;
        render(): JSX.Element;
    }
    export class InputRange extends BasicInput<InputProps<number>> {
        protected onInput(event: any): void;
        protected onChange(event: any): void;
        render(): JSX.Element;
    }
    export interface InputTextBoxProps<T> extends JSX.HTMLAttributes<HTMLTextAreaElement> {
        $: $Proxy<T>;
        handleTabs?: boolean;
        value?: undefined;
        onInput?: undefined;
    }
    export class InputTextBox extends BasicInput<InputTextBoxProps<string>> {
        protected onInput(event: any): void;
        protected onChange(event: any): void;
        protected onKeyDown(event: KeyboardEvent, input: HTMLTextAreaElement): boolean;
        render(): JSX.Element;
    }
    export class InputCheckbox extends BasicInput<InputProps<boolean>> {
        protected onInput(event: any): void;
        protected onChange(event: any): void;
        render(): JSX.Element;
    }
    export class BasicText extends BasicInput<InputProps<string | number>> {
        render(): JSX.Element;
    }
    export class FixedNumber extends BasicInput<InputProps<number> & {
        digits: number;
        multiplier?: number;
        prefix?: string;
        postfix?: string;
    }> {
        render(): JSX.Element;
    }
    export interface SelectProps<T> extends JSX.HTMLAttributes<HTMLSelectElement> {
        $: $Proxy<T>;
        value?: undefined;
        onInput?: undefined;
        onChange?: undefined;
    }
    export class InputSelect extends BasicInput<{
        values: {
            [K: string]: string;
        } | string[];
    } & SelectProps<string>> {
        protected onInput(event: any): void;
        protected onChange(event: any): void;
        render(): JSX.Element;
    }
    export class InputFile extends BasicComponent<{
        onFile: (file: File | undefined) => void;
    } & JSX.HTMLAttributes<HTMLInputElement>> {
        protected onChange(event: {
            target: HTMLInputElement;
        }): void;
        render(): JSX.Element;
    }
    export class InputFiles extends BasicComponent<{
        onFiles: (files: File[] | undefined) => void;
    } & JSX.HTMLAttributes<HTMLInputElement>> {
        protected onChange(event: {
            target: HTMLInputElement;
        }): void;
        render(): JSX.Element;
    }
}
declare module "Components/JSONView" {
    import { BasicComponent } from "Components/BasicComponent";
    import "./JSONView.scss";
    export class JSONView extends BasicComponent<{
        json: any;
    }> {
        willReceiveProps(): void;
        render(): JSX.Element;
    }
}
declare module "Components/Tooltip" {
    import { VNode, Component, ComponentChildren } from "TSX/TSX";
    import { BasicComponent } from "Components/BasicComponent";
    import "./Tooltip.scss";
    export class Tooltip extends BasicComponent<{
        [Component.Children]?: ComponentChildren | object;
    } & JSX.HTMLAttributes<HTMLDivElement>> {
        render(): JSX.Element;
    }
    export class TooltipProvider extends BasicComponent<{
        renderTooltip: () => VNode;
    } & JSX.HTMLAttributes<HTMLDivElement>> {
        protected willReceiveProps(): void;
        private tooltip;
        private onMouseEnter;
        private onMouseLeave;
        render(): JSX.Element;
    }
}
declare module "Components/ListContainer" {
    import { BasicEvent } from "@h4x/common";
    export class ListContainer<T> {
        private internal;
        constructor(...init: T[]);
        get size(): number;
        set(list: T[]): void;
        get(index: number): T;
        add(value: T): number;
        remove(index: number): number;
        removeValue(value: T): number;
        each(callback: (value: T, index: number) => void): void;
        map(callback: (value: T, index: number) => void): void[];
        onChanged: BasicEvent<() => void, () => void>;
        onRemove: BasicEvent<(value: T, index: number) => void, (value: T, index: number) => void>;
    }
}
declare module "Components/ListSelect" {
    import { BasicEvent } from "@h4x/common";
    import { ListContainer } from "Components/ListContainer";
    import { BasicComponent } from "Components/BasicComponent";
    import "./ListSelect.scss";
    export class SelectContainer extends ListContainer<string | ({
        [K: string]: any;
        label: string;
        labelClass?: string | {
            [K: string]: boolean | undefined;
        };
    })> {
        onSelected: BasicEvent<(index: number) => void, (index: number) => void>;
        private currentIndex;
        get index(): number;
        constructor(...init: (string | ({
            [K: string]: any;
            label: string;
            labelClass?: string | {
                [K: string]: boolean | undefined;
            };
        }))[]);
        linkValue<C, F extends keyof C = keyof C>(variable: C, field: F): boolean;
        update(index: number): void;
        prev(): void;
        next(): void;
    }
    export interface AppProps extends JSX.HTMLAttributes<HTMLDivElement> {
        options: SelectContainer;
        children?: undefined;
    }
    export class ListSelect extends BasicComponent<AppProps> {
        val: number;
        private uuid;
        private options;
        init(): void;
        willReceiveProps(nextProps: AppProps): void;
        render(): JSX.Element;
    }
}
declare module "Components/Fragment" {
    import { Component, TSX } from "TSX/TSX";
    import { BasicComponent } from "Components/BasicComponent";
    const $remove: unique symbol;
    const $rendered: unique symbol;
    const $attach: unique symbol;
    export abstract class Fragment<T> extends BasicComponent<T & {
        fragmentList: Fragment.AbstractList<Fragment<T>>;
    }> {
        [$rendered]: TSX.RenderedComponent<Fragment<T>>;
        remove(): void;
        onDetached(): void;
    }
    export namespace Fragment {
        abstract class AbstractList<T extends Fragment<any>> {
            protected component: Container<T>;
            abstract [Symbol.iterator](): Iterator<TSX.RenderedComponent<T>>;
            instances(): Generator<T, void, unknown>;
            abstract get first(): TSX.RenderedComponent<T> | undefined;
            abstract get firstInstance(): T | undefined;
            abstract get last(): TSX.RenderedComponent<T> | undefined;
            abstract get lastInstance(): T | undefined;
            abstract get length(): number;
            abstract indexOf(rendered: TSX.RenderedComponent<T> | T): number;
            protected createFragment<K extends T>(FragmentClass: new () => K, args: Omit<TSX.Props<K>, "fragmentList">): {
                rendered: TSX.RenderedComponent<T>;
                instance: K;
            };
            [$attach](component: Container<T>): void;
            abstract [$remove](rendered: TSX.RenderedComponent<T>): boolean;
        }
        class List<T extends Fragment<any>> extends AbstractList<T> {
            private readonly fragments;
            get length(): number;
            get first(): TSX.RenderedComponent<T>;
            get firstInstance(): T;
            get last(): TSX.RenderedComponent<T>;
            get lastInstance(): T;
            [Symbol.iterator](): Generator<TSX.RenderedComponent<T>, void, unknown>;
            get(at: number): TSX.RenderedComponent<T>;
            getInstance(at: number): T;
            indexOf(rendered: TSX.RenderedComponent<T> | T): number;
            /** Add fragment before `at` or at the end */
            addFragment<K extends T>(FragmentClass: new () => K, args: Omit<TSX.Props<K>, "fragmentList">, at?: number | T): K;
            moveFragment(rendered: TSX.RenderedComponent<T>, to?: number): void;
            detachFragment(rendered: TSX.RenderedComponent<T>): boolean;
            detachAll(): void;
            destroyAll(): void;
            [$attach](component: Container<T>): void;
            /** internal */
            [$remove](rendered: TSX.RenderedComponent<T>): boolean;
        }
        class Single<T extends Fragment<any>> extends AbstractList<T> {
            private fragment?;
            get length(): 1 | 0;
            get first(): TSX.RenderedComponent<T>;
            get firstInstance(): T;
            get last(): TSX.RenderedComponent<T>;
            get lastInstance(): T;
            [Symbol.iterator](): Generator<TSX.RenderedComponent<T>, void, unknown>;
            indexOf(rendered: TSX.RenderedComponent<T> | T): 0 | -1;
            set<K extends T>(FragmentClass: new () => K, args: Omit<TSX.Props<K>, "fragmentList">): K;
            attach(rendered: TSX.RenderedComponent<T>): void;
            get(): TSX.RenderedComponent<T>;
            get instance(): T;
            detach(): void;
            destroy(): void;
            /** internal */
            [$remove](rendered: TSX.RenderedComponent<T>): boolean;
        }
        namespace Container {
            interface Props<T extends Fragment<any>> extends JSX.HTMLAttributes<HTMLDivElement> {
                nodeType?: string;
                fragmentList: AbstractList<T>;
                key: string;
                [Component.Children]?: undefined;
            }
        }
        class Container<T extends Fragment<any>> extends BasicComponent<Container.Props<T>> {
            private container;
            attachContainer(instance: HTMLDivElement): void;
            attachFragment(rendered: TSX.RenderedComponent<T>, at?: number | Node): void;
            render(): JSX.Element;
        }
    }
}
declare module "Components/ConfigEditor/Containers/ObjectContainer" {
    import { BasicComponent } from "Components/BasicComponent";
    import { BasicEvent } from "@h4x/common";
    import { ObjectType } from "Components/ConfigEditor/CEUtils";
    export class ObjectContainer extends BasicComponent<{
        schema: ObjectType;
        data: any;
        onUpdate: BasicEvent<any>;
    }> {
        willReceiveProps(): void;
        render(): JSX.Element;
    }
}
declare module "Components/ConfigEditor/Containers/MapContainer" {
    import { BasicComponent } from "Components/BasicComponent";
    import { BasicEvent } from "@h4x/common";
    import { MapType } from "Components/ConfigEditor/CEUtils";
    export class MapContainer extends BasicComponent<{
        schema: MapType;
        data: any;
        onUpdate: BasicEvent<any>;
    }> {
        willReceiveProps(): void;
        newName: string;
        private addNew;
        private onDeleted;
        render(): JSX.Element;
    }
}
declare module "Components/ConfigEditor/CEValidator" {
    import { NumberType, StringType } from "Components/ConfigEditor/CEUtils";
    export enum ValidatorType {
        Error = 0,
        Warning = 1,
        Info = 2
    }
    export type ValidatorOutput = {
        errors: string[];
        warnings: string[];
        info: string[];
    };
    export class Validator {
        static boolean(schema: NumberType, value: boolean, container: any, key: string | number): ValidatorOutput;
        static text(schema: StringType, value: string, container: any, key: string | number): ValidatorOutput;
        static number(schema: NumberType, value: number, container: any, key: string | number): ValidatorOutput;
        private static numberRange;
        private static required;
    }
}
declare module "Components/ConfigEditor/Containers/BooleanContainer" {
    import { BasicComponent } from "Components/BasicComponent";
    import { BasicEvent } from "@h4x/common";
    export class BooleanContainer extends BasicComponent<{
        schema: any;
        data: any;
        key: any;
        onUpdate: BasicEvent<any>;
        value?: undefined;
        onInput?: undefined;
    }> {
        init(): void;
        willReceiveProps(): void;
        get(): boolean;
        set(value: boolean): void;
        private onInput;
        private validatorData;
        private validate;
        render(): JSX.Element;
    }
}
declare module "Components/ConfigEditor/Containers/NumberContainer" {
    import { BasicComponent } from "Components/BasicComponent";
    import { BasicEvent } from "@h4x/common";
    import { NumberType } from "Components/ConfigEditor/CEUtils";
    export class NumberContainer extends BasicComponent<{
        schema: NumberType;
        data: any;
        key: any;
        onUpdate: BasicEvent<any>;
        value?: undefined;
        onInput?: undefined;
    }> {
        init(): void;
        willReceiveProps(): void;
        get(): number;
        set(value: number): void;
        private onInput;
        private validatorData;
        private validate;
        render(): JSX.Element;
    }
}
declare module "Components/ConfigEditor/Containers/TextContainer" {
    import { BasicComponent } from "Components/BasicComponent";
    import { BasicEvent } from "@h4x/common";
    import { StringType } from "Components/ConfigEditor/CEUtils";
    export class TextContainer extends BasicComponent<{
        schema: StringType;
        data: any;
        key: any;
        onUpdate: BasicEvent<any>;
        value?: undefined;
        onInput?: undefined;
    }> {
        init(): void;
        willReceiveProps(): void;
        get(): string;
        set(value: string): void;
        private onInput;
        private validatorData;
        private validate;
        render(): JSX.Element;
    }
}
declare module "Components/ConfigEditor/CEUtils" {
    import { BasicEvent } from "@h4x/common";
    import { ValidatorOutput } from "Components/ConfigEditor/CEValidator";
    export enum Type {
        Unknown = "unknown",
        Boolean = "boolean",
        Number = "number",
        String = "string",
        Object = "object",
        Array = "array",
        Map = "map"
    }
    export interface ElementType {
        type: Type;
        optional?: boolean;
    }
    export interface BooleanType extends ElementType {
        type: Type.Boolean;
        default?: boolean;
    }
    export interface NumberType extends ElementType {
        type: Type.Number;
        default?: number;
        min?: number;
        max?: number;
    }
    export interface StringType extends ElementType {
        type: Type.String;
        default?: string;
        multiline?: boolean;
        options?: string[];
    }
    export interface ObjectType extends ElementType {
        type: Type.Object;
        properties: {
            [K: string]: ElementTypes;
        };
    }
    export interface ArrayType extends ElementType {
        type: Type.Array;
        elements: ElementTypes;
    }
    export interface MapType extends ElementType {
        type: Type.Map;
        elements: ElementTypes;
    }
    export type ElementTypes = BooleanType | NumberType | StringType | ObjectType | ArrayType | MapType;
    export type ContainerTypes = ObjectType | ArrayType | MapType;
    export namespace CEUtils {
        function getNew(schema: ElementTypes): any;
        function renderInput(schema: any, data: any, key: string, onUpdate: BasicEvent<any>): JSX.Element;
        function parseInfo(data: ValidatorOutput): JSX.Element[];
    }
}
declare module "Components/ConfigEditor/Containers/ArrayContainer" {
    import { BasicComponent } from "Components/BasicComponent";
    import { BasicEvent } from "@h4x/common";
    import { ArrayType } from "Components/ConfigEditor/CEUtils";
    export class ArrayContainer extends BasicComponent<{
        schema: ArrayType;
        data: any[];
        onUpdate: BasicEvent<any>;
    }> {
        willReceiveProps(): void;
        private addNew;
        private onDeleted;
        render(): JSX.Element;
    }
}
declare module "Components/ConfigEditor/Containers/RootContainer" {
    import { BasicComponent } from "Components/BasicComponent";
    import { BasicEvent } from "@h4x/common";
    import { ContainerTypes } from "Components/ConfigEditor/CEUtils";
    export class RootContainer extends BasicComponent<{
        name: string;
        schema: ContainerTypes;
        data: any;
        onUpdate: BasicEvent<any>;
    } & JSX.HTMLAttributes<HTMLDivElement>> {
        willReceiveProps(): void;
        render(): JSX.Element;
    }
}
declare module "Components/ConfigEditor/ConfigEditor" {
    import { BasicComponent } from "Components/BasicComponent";
    import "./ConfigEditor.scss";
    export class ConfigEditor extends BasicComponent {
        json: string;
        private onUpdate;
        private data;
        private initLink;
        render(): JSX.Element;
    }
}
declare module "esm" {
    export * from "TSX/TSX";
    export * from "TSX/Overlay";
    export * from "TSX/global";
    export * from "Components/BasicComponent";
    export * from "Components/FormComponents";
    export * from "Components/JSONView";
    export * from "Components/Tooltip";
    export * from "Components/ListContainer";
    export * from "Components/ListSelect";
    export * from "Components/Fragment";
    export * from "Components/ConfigEditor/Containers/ArrayContainer";
    export * from "Components/ConfigEditor/Containers/BooleanContainer";
    export * from "Components/ConfigEditor/Containers/MapContainer";
    export * from "Components/ConfigEditor/Containers/NumberContainer";
    export * from "Components/ConfigEditor/Containers/ObjectContainer";
    export * from "Components/ConfigEditor/Containers/RootContainer";
    export * from "Components/ConfigEditor/Containers/TextContainer";
    export * from "Components/ConfigEditor/ConfigEditor";
    export * from "Components/ConfigEditor/CEUtils";
    export * from "Components/ConfigEditor/CEValidator";
}
